package uibk.applets.riemann3d;

import java.awt.Color;
import uibk.mtk.draw2d.base.MathPanel2D;
import uibk.mtk.draw2d.objects.RiemannSum2d;
import uibk.mtk.draw3d.axescube3d.AxesCube3D;
import uibk.mtk.draw3d.base.RMathPanel3D;
import uibk.mtk.draw3d.base.Scene3D;
import uibk.mtk.draw3d.objects.surfaceext.SurfaceWithLines;
import uibk.mtk.draw3d.rasterizer.Rasterizer;
import uibk.mtk.lang.Messages;
import uibk.mtk.math.functions.ParsedFunction2D;
import uibk.mtk.swing.appletbase.AppletBase;
import uibk.mtk.swing.scene3d.Panel2VisibleSurface;
import uibk.mtk.geom.geom2d.Punkt2D;

/**
 * Applet zur 3D-Visualisierung von Funktionen von R^2->R. und die Riemannsumme
 * des Volumens *
 * <p>
 * Copyright: Institut fr Grundlagen der Bauingenieurwissenschaften, 2006
 * </p>
 * 
 * @author Andreas Rudigier (csae7164@uibk.ac.at)
 */
public class AppletRiemann3D extends AppletBase {

	/** RMathPanel3D der die Flche verwaltet */
	RMathPanel3D mathpanel3d = new RMathPanel3D();

	/** MathPanel2D der das Gitter verwaltet */
	MathPanel2D mathpanel2d = new MathPanel2D();

	protected transient RiemannSum2d riemannvisx = new RiemannSum2d();
	protected transient RiemannSum2d riemannvisy = new RiemannSum2d();

	/** die Funktion R^2->R */
	ParsedFunction2D function;

	/** Panel zur Sichtbarkeit der Flchen */
	Panel2VisibleSurface visible;
	
	/** Die Flchen */
	SurfaceWithLines[] surface3d = new SurfaceWithLines[2];

	/** Variablen in denen die Funktion definiert sein muss */
	static final String[] VARS = { "x", "y" }; //$NON-NLS-1$ //$NON-NLS-2$

	/** maximale Gre des Gitters ber dem die Flche gezeichnet wird */
	static final int GRIDMAX = 10;

	/** minimale Gre des Gitters ber dem die Flche gezeichnet wird */
	static final int GRIDMIN = 2;

	/** Standardgre des Gitters ber dem die Flche gezeichnet wird */
	static final int GRIDDEFAULT = 10;
	
	/** Panel zum Gitter des Definitionsbereichs */
	PanelChoose panelchoose=new PanelChoose(this);

	/** Panel zum Auswhlen der partiellen Ableitungen */
	PanelPartialDerivates panelpartialderivs;

	/** Panel zum Eingeben und Zeichnen der Flche */
	PanelSurface panelsurface;

	/** Panel zur Ausgabe der Integration */
	PanelOutput paneloutput;
	
	/** die Achsenbox */
	AxesCube3D axescube3d;

	/** die Gitterpunkte der Partionierung */
	Punkt2D[][] gridpoints;
	
	/** die Knotenpunkte der Quadratformel */
	double[][][] knotpoints;
	
	/** die Gewichte der Quadratformel */
	double[][] weigths;

	/** Erstmalige Berechnung des exakten Integrals */
	boolean first = true;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.applets.riemann3d.messages";//$NON-NLS-1$

	/** Initialisierung der Komponenten */
	protected void initComponents() {
		super.initComponents();
		super.panelnav.enableTheoryButton(false);
		super.panelnav.setStrcopyright(Messages.getString(BUNDLE_NAME,
				"AppletRiemann3D.1"));
		super.panelnav.setStrauthor(Messages.getString(BUNDLE_NAME,
		"AppletRiemann3D.2"));
		super.panelnav.setHome("../indexAnd.html");
		

		//Gitter unsichtbar
		riemannvisx.setAxes(true);
		riemannvisx.setNumberSubDivisions(5);
		riemannvisx.setColor(Color.WHITE);
		riemannvisx.setVisible(true);
		riemannvisy.setAxes(false);
		riemannvisy.setNumberSubDivisions(5);
		riemannvisy.setColor(Color.WHITE);
		riemannvisy.setVisible(true);
		//Visualiserung und Achsen zu MathPanel hinzuf?gen
		mathpanel2d.add(riemannvisx);
		mathpanel2d.add(riemannvisy);
		
		//Drag and drop einschalten
		mathpanel2d.setListenForDragAndDrop(true);
		mathpanel2d.setListenForClicks(true);
		
		surface3d[0] = new SurfaceWithLines();
		surface3d[0].setVisible(false);
		surface3d[1] = new SurfaceWithLines();
		surface3d[1].setVisible(false);
		surface3d[1].setVisibility(true);

		mathpanel3d.getScene3D().setScalingMode(Scene3D.SCALE_INDEPENDENT);
		axescube3d = new AxesCube3D();
		axescube3d.enableLabels(true);
		axescube3d.setLabels(VARS[0], VARS[1], "z"); //$NON-NLS-1$
		mathpanel3d.add(surface3d[0]);
		mathpanel3d.add(surface3d[1]);
		mathpanel3d.add(axescube3d);
		mathpanel3d.getRasterizer().setShadingMode(Rasterizer.GOUROUD);

		super.setMainPanel(mathpanel3d);
		super.setControlPanel(new PanelCommand(this));

	}

	public static void main(String[] args) {
		runmain(new AppletRiemann3D(), Messages.getString(BUNDLE_NAME,
				"AppletRiemann3D.3")); //$NON-NLS-1$
	}

}