package uibk.applets.riemann3d;

import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JTabbedPane;

import uibk.mtk.lang.Messages;
import uibk.mtk.swing.appletbase.PanelLoadExample;
import uibk.mtk.swing.base.MPanel;
import uibk.mtk.swing.base.TextField;
import uibk.mtk.swing.scene3d.Panel2Surface3DStyle;
import uibk.mtk.swing.scene3d.Panel2VisibleSurface;
import uibk.mtk.swing.scene3d.PanelAxes3DStyle;
import uibk.mtk.swing.scene3d.Panel2Coloring;
import uibk.mtk.swing.scene3d.PanelRendering;
import uibk.mtk.swing.scene3d.PanelScale3D;

import uibk.mtk.swing.scene3d.PanelView3D;

/**
 * Panel zum Steuern des Applets
 * 
 * <p>
 * Copyright: Institut fr Grundlagen der Bauingenieurwissenschaften, 2006
 * </p>
 * 
 * @author Andreas Rudigier (csae7164@uibk.ac.at)
 */
class PanelCommand extends MPanel {

	/** Referenz auf die Hauptklasse */
	private AppletRiemann3D main;

	public PanelCommand(AppletRiemann3D main) {

		this.main = main;
		initComponents();

	}

	/** Initialisieren der Komponenten */
	private void initComponents() {
		// Rahmen und Layout
		setBorder(BorderFactory.createLineBorder(Color.black));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		main.visible = new Panel2VisibleSurface(main.surface3d,
				main.mathpanel3d);
		main.visible.setEnabled(false);

		MPanel tab1 = new MPanel();
		MPanel tab2 = new MPanel();
		MPanel tab3 = new MPanel();
		MPanel tab4 = new MPanel();
		MPanel tab5 = new MPanel();

		tab1.setLayout(new BoxLayout(tab1, BoxLayout.Y_AXIS));
		tab2.setLayout(new BoxLayout(tab2, BoxLayout.Y_AXIS));
		tab3.setLayout(new BoxLayout(tab3, BoxLayout.Y_AXIS));
		tab4.setLayout(new BoxLayout(tab4, BoxLayout.Y_AXIS));
		tab5.setLayout(new BoxLayout(tab5, BoxLayout.Y_AXIS));

		// Panel zum Erzeugen der Flche
		main.panelsurface = new PanelSurface(main);
		tab1.add(createPanelLoadExamples());
		tab1.add(main.panelsurface);

		// Panel fr Projektionsoptionen, zum Skalieren, zum Auswhlen
		// des Stils der Flche und der Frbungs und Renderoptionen
		// hinzufgen
		tab2.add(new PanelView3D(main.mathpanel3d));
		tab3.add(new PanelScale3D(main.mathpanel3d));
		tab3.add(new PanelAxes3DStyle(main.axescube3d, main.mathpanel3d));
		tab4.add(main.visible);
		tab4.add(new Panel2Surface3DStyle(main.surface3d, main.mathpanel3d));
		tab4.add(new Panel2Coloring(main.mathpanel3d, main.surface3d));
		tab4.add(new PanelRendering(main.mathpanel3d));

		// Panel zum Steuern der partiellen Ableitungen
		main.panelpartialderivs = new PanelPartialDerivates(main);
		main.panelpartialderivs.setEnabled(false);
		main.paneloutput = new PanelOutput();

		tab5.add(main.panelpartialderivs);
		tab5.add(main.paneloutput);

		// Erzeugen eines Panels mit Reitern
		JTabbedPane tabbedpane = new JTabbedPane();

		tabbedpane
				.addTab(
						Messages.getString(AppletRiemann3D.BUNDLE_NAME,
								"PanelCommand.0"), null, tab1, Messages.getString(AppletRiemann3D.BUNDLE_NAME, "PanelCommand.1")); //$NON-NLS-1$ //$NON-NLS-2$
		tabbedpane
				.addTab(
						Messages.getString(AppletRiemann3D.BUNDLE_NAME,
								"PanelCommand.2"), null, tab2, Messages.getString(AppletRiemann3D.BUNDLE_NAME, "PanelCommand.3")); //$NON-NLS-1$ //$NON-NLS-2$
		tabbedpane.addTab(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelCommand.4"), //$NON-NLS-1$
				null, tab3, Messages.getString(AppletRiemann3D.BUNDLE_NAME,
						"PanelCommand.5")); //$NON-NLS-1$
		tabbedpane
				.addTab(
						Messages.getString(AppletRiemann3D.BUNDLE_NAME,
								"PanelCommand.6"), null, tab4, Messages.getString(AppletRiemann3D.BUNDLE_NAME, "PanelCommand.7")); //$NON-NLS-1$ //$NON-NLS-2$
		tabbedpane.addTab(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelCommand.8"), //$NON-NLS-1$
				null, tab5, Messages.getString(AppletRiemann3D.BUNDLE_NAME,
						"PanelCommand.9")); //$NON-NLS-1$

		add(tabbedpane);
	}

	/** Erzeugt den Panel zum Laden der Beispiele */
	MPanel createPanelLoadExamples() {
		String[] options = { "x^2+y^2", "x^3-y^3", "sin(x)cos(y)", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				"exp(-(x^2+y^2))", "(x^2-y^2)*sin(x)" }; //$NON-NLS-1$ //$NON-NLS-2$
		TextField[] textfields = { main.panelsurface.textFormula,
				main.panelsurface.txtIntervalX, main.panelsurface.txtIntervalY };
		String[][] values = { { "x^2+y^2", "[-1,5]", "[-1,0.5]" }, //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				{ "x^3-y^3", "[-1,-0.3]", "[-1,0.4]" }, //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				{ "sin(x)cos(y)", "[-5,5]", "[-5,0.2]" }, //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				{ "exp(-(x^2+y^2))", "[-1.5,-0.5]", "[-1.5,0.2]" }, //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				{ "(x^2-y^2)*sin(x)", "[-4,1]", "[-4,-.05]" }, }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		return new PanelLoadExample(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelCommand.30"), //$NON-NLS-1$
				Messages.getString(AppletRiemann3D.BUNDLE_NAME,
						"PanelCommand.31"), //$NON-NLS-1$
				options, textfields, values);

	}

}