package uibk.applets.riemann3d;

import Jama.Matrix;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.util.Random;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import uibk.mtk.lang.ComputationException;
import uibk.applets.riemann3d.PanelChoose;
import uibk.mtk.lang.Messages;
import uibk.mtk.math.Interval;
import uibk.mtk.math.integration.GaussIntegration2D;
import uibk.mtk.math.NewtonMethod;
import uibk.mtk.math.NewtonMethod2D;
import uibk.mtk.swing.base.TitledPanel;
import uibk.mtk.util.Grid2D;
import uibk.mtk.draw3d.objects.surface3d.Mesh;
import uibk.mtk.draw3d.objects.surface3d.Triangle;
import uibk.mtk.draw3d.objects.surface3d.Vertex3D;
import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.geom.geom2d.CoordinateRect2D;
import uibk.mtk.geom.geom3d.CoordinateBox3D;

/**
 * Panel zum Auswhlen in wieviele Punkten die Funktion in jedem Quader
 * ausgewertet wird, ber die dann der Mittelwert in diesem Quader gebildet
 * wird. Auerdem kann die Anordnung quidistant oder zufllig sein. Man kann
 * die Punkte und auch die Quadergrenzen verschieben.
 * 
 * <p>
 * Copyright: Institut fr Grundlagen der Bauingenieurwissenschaften, 2006
 * </p>
 * 
 * @author Andreas Rudigier (csae7164@uibk.ac.at)
 */
public class PanelPartialDerivates extends TitledPanel implements
		ActionListener {

	/** Referenz auf die Hauptklasse */
	AppletRiemann3D main;

	/** Option Gauߴsche zweidimensionale Quadraturformel */
	JRadioButton QuadraturFormel;

	/** Option ein zuflliger Riemannpunkt */
	JRadioButton zufllig1;

	/** Option wechselt Gittervisualisierung mit der Flchenvisualisierung */
	JRadioButton zoom;

	/** Aktiviert Ober- und Untersumme */
	JRadioButton oberunter;

	/** Obersumme wird dargestellt */
	JRadioButton ober;

	/** Untersumme wird dargestellt */
	JRadioButton unter;

	/** Slider zur Anpassung der Stufe der QF */
	JSlider sliderStep;

	/** Label mit der aktuellen Gre des Gitters */
	JLabel labelstep;

	/** Slider mit der aktuellen Gre des Gitters */
	JSlider slidergitter;

	/** Label mit der aktuellen Gre des Gitters */
	JLabel labelgitter;

	/**
	 * Array f?r die durch die Funktion und das Gitter definierten Vertices
	 * 
	 */
	Vertex3D[] vertices;

	/**
	 * Funktionswerte in jedem Rechteck
	 */
	double[][] value;

	/**
	 * X-Komponente an der die Funktion ausgewertet wird
	 */
	double valuex;

	/**
	 * Y-Komponente an der die Funktion ausgewertet wird
	 */
	double valuey;

	/**
	 * Obersummenwert
	 */
	double valueup = Double.NaN;

	/**
	 * Untersummenwert
	 */
	double valuedown = Double.NaN;

	/**
	 * Rechteck indem das Maximum oder das Minimum gesucht wird
	 */
	CoordinateRect2D rechteck = new CoordinateRect2D();

	/**
	 * Rechteck indem das Newtonverfahren die Nullstelle der Ableitung sucht
	 */
	CoordinateRect2D rechteckit = new CoordinateRect2D();

	/**
	 * 
	 * Gitter der Riemannsumme
	 */
	Punkt2D[][] grid;

	/**
	 * 
	 * Gitter der Newtoniteration fr maximum und minimum
	 */
	Punkt2D[][] gridnewton;

	/**
	 * eindimensionales Newton-Verfahren
	 */
	NewtonMethod ein = new NewtonMethod();

	/** Button zum Starten des Plottens */
	JButton buttonCalc;

	/** Erzeugt den Panel */
	public PanelPartialDerivates(AppletRiemann3D main) {
		super(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelPartialDerivates.0")); //$NON-NLS-1$
		this.main = main;
		initComponents();

	}

	/** Berechnet das Minimum einer Funktion auf dem Rechteck */
	private void getmaxmin(int i, int j) {
		double a[] = new double[4];
		double defect = 1;
		double f;
		Matrix min = new Matrix(1, 1);
		NewtonMethod2D zweimin = new NewtonMethod2D();

		// Tiefpunkt und Hochpunkt in dem Rechteck berechnen
		zweimin.setFunction(main.function);
		for (int k = 0; k < gridnewton.length; k++) {
			for (int m = 0; m < gridnewton[0].length; m++) {
				zweimin.setX0(gridnewton[k][m].x);
				zweimin.setY0(gridnewton[k][m].y);
				min = zweimin.getXY();
				while (defect > 1E-6
						&& rechteckit.contains(min.get(0, 0), min.get(1, 0))) {
					try {
						min = zweimin.nextApproximation();
						defect = zweimin.getDefect();
					} catch (ComputationException e) {
						defect = 0;
					}
				}
				if (rechteck.contains(min.get(0, 0), min.get(1, 0))) {
					valuex = min.get(0, 0);
					valuey = min.get(1, 0);
					valueup = valueup > main.function.getValue(valuex, valuey) ? valueup
							: main.function.getValue(valuex, valuey);
					valuedown = valuedown < main.function.getValue(valuex,
							valuey) ? valuedown : main.function.getValue(
							valuex, valuey);
				}

			}
		}
		ein.setFunction2D(main.function);

		// linke Seite
		for (int m = 0; m < gridnewton.length; m++) {
			defect = 1;
			f = gridnewton[m][0].x;
			ein.setX0(f);
			while (defect > 1E-8 && rechteckit.contains(f, grid[i][j].y)) {
				try {
					f = ein.nextApproximation(grid[i][j].y, 2);
					defect = ein.getDefect();
				} catch (ComputationException e) {
					defect = 0;
				}
			}
			if (rechteck.contains(f, grid[i][j].y)) {
				valueup = valueup > main.function.getValue(grid[i][j].x, f) ? valueup
						: main.function.getValue(grid[i][j].x, f);
				valuedown = valuedown < main.function.getValue(f, grid[i][j].y) ? valuedown
						: main.function.getValue(f, grid[i][j].y);
			}
		}

		// obere Seite
		for (int m = 0; m < gridnewton[0].length; m++) {
			defect = 1;
			f = gridnewton[0][m].y;
			ein.setX0(f);
			while (defect > 1E-6 && rechteckit.contains(grid[i][j].x, f)) {
				try {
					f = ein.nextApproximation(grid[i][j].x, 1);
					defect = ein.getDefect();
				} catch (ComputationException e) {
					defect = 0;
				}
			}
			if (rechteck.contains(grid[i][j].x, f)) {
				valueup = valueup > main.function.getValue(grid[i][j].x, f) ? valueup
						: main.function.getValue(grid[i][j].x, f);
				valuedown = valuedown < main.function.getValue(grid[i][j].x, f) ? valuedown
						: main.function.getValue(grid[i][j].x, f);
			}

		}

		// rechte Seite
		for (int m = 0; m < gridnewton.length; m++) {
			defect = 1;
			f = gridnewton[m][gridnewton[0].length - 1].x;
			ein.setX0(f);
			while (defect > 1E-6 && rechteckit.contains(f, grid[i + 1][j].y)) {
				try {
					f = ein.nextApproximation(grid[i + 1][j].y, 2);
					defect = ein.getDefect();
				} catch (ComputationException e) {
					defect = 0;
				}

			}
			if (rechteck.contains(f, grid[i + 1][j].y)) {
				valueup = valueup > main.function.getValue(grid[i][j].x, f) ? valueup
						: main.function.getValue(grid[i][j].x, f);
				valuedown = valuedown < main.function.getValue(f,
						grid[i + 1][j].y) ? valuedown : main.function.getValue(
						f, grid[i + 1][j].y);
			}
		}

		// untere Seite
		for (int m = 0; m < gridnewton[0].length; m++) {
			defect = 1;
			f = gridnewton[gridnewton.length - 1][m].y;
			ein.setX0(f);
			while (defect > 1E-6 && rechteckit.contains(grid[i][j + 1].x, f)) {
				try {
					f = ein.nextApproximation(grid[i][j + 1].x, 1);
					defect = ein.getDefect();
				} catch (ComputationException e) {
					defect = 0;
				}

			}
			if (rechteck.contains(grid[i][j + 1].x, f)) {
				valueup = valueup > main.function.getValue(grid[i][j].x, f) ? valueup
						: main.function.getValue(grid[i][j].x, f);
				valuedown = valuedown < main.function.getValue(
						grid[i][j + 1].x, f) ? valuedown : main.function
						.getValue(grid[i][j + 1].x, f);
			}

		}
		// Funktionswerte der Ecken
		a[0] = main.function.getValue(grid[i][j].x, grid[i][j].y);
		a[1] = main.function.getValue(grid[i + 1][j].x, grid[i + 1][j].y);
		a[2] = main.function.getValue(grid[i][j + 1].x, grid[i][j + 1].y);
		a[3] = main.function.getValue(grid[i + 1][j + 1].x,
				grid[i + 1][j + 1].y);
		for (int index = 0; index < 4; index++)
			if (Double.isNaN(valuedown)) {
				valuedown = a[index];
				valueup = a[index];
			} else {
				valuedown = valuedown < a[index] ? valuedown : a[index];
				valueup = valueup > a[index] ? valueup : a[index];
			}

		return;
	}

	/** Setzt das Diagramm in das MainPanel von Applet Base */
	public void setselected() {
		if (!main.panelpartialderivs.isEnabled()) {
			main.panelpartialderivs.setEnabled(true);
			main.panelpartialderivs.QuadraturFormel.setSelected(true);
			main.panelpartialderivs.ober.setEnabled(false);
			main.panelpartialderivs.unter.setEnabled(false);
		} else {
			if (main.panelpartialderivs.QuadraturFormel.isSelected()) {
				main.panelpartialderivs.sliderStep.setEnabled(true);
				main.panelpartialderivs.ober.setEnabled(false);
				main.panelpartialderivs.unter.setEnabled(false);
			}
			if (main.panelpartialderivs.zufllig1.isSelected()) {
				main.panelpartialderivs.sliderStep.setEnabled(false);
				main.panelpartialderivs.ober.setEnabled(false);
				main.panelpartialderivs.unter.setEnabled(false);
			}
			if (main.panelpartialderivs.oberunter.isSelected()) {
				main.panelpartialderivs.sliderStep.setEnabled(false);
				main.panelpartialderivs.ober.setEnabled(true);
				main.panelpartialderivs.unter.setEnabled(true);
			}
		}
	}

	/** Setzt das Diagramm in das MainPanel von Applet Base */
	private void zoom() throws Exception {
		// Alles unsichtbar machen
		main.mathpanel3d.setVisible(false);
		main.mathpanel2d.setVisible(false);
		// Zoomen
		if (zoom.isSelected()) {
			main.setMainPanel(main.mathpanel2d);
			main.mathpanel3d.setVisible(false);
			main.mathpanel2d.setVisible(true);
		} else {
			main.mathpanel2d.setVisible(false);
			main.mathpanel3d.setVisible(true);
			main.setMainPanel(main.mathpanel3d);
		}
	}

	void creategrid()throws Exception{
		Interval intervalx = main.panelsurface.intervalreal;
		Interval intervaly = main.panelsurface.intervalimag;
		main.riemannvisx.setInterval(intervalx);
		main.riemannvisy.setInterval(intervaly);
		main.riemannvisx.setIntervalimag(intervaly);
		main.riemannvisy.setIntervalimag(intervalx);
		main.riemannvisx.create();
		main.riemannvisy.create();
		main.mathpanel2d.getScene2d().setXInterval(intervalx);
		main.mathpanel2d.getScene2d().setYInterval(intervaly);
		
		}
	/** Berechnet und zeichnet die Treppenfunktion */
	private void compute() {
		double sum = 0;
		double sumup = 0;
		double sumdown = 0;
		int s = main.panelpartialderivs.sliderStep.getValue();
		int lengthx = main.panelpartialderivs.slidergitter.getValue();
		int lengthy = lengthx;
		Interval x = main.panelsurface.intervalreal;
		Interval y = main.panelsurface.intervalimag;
		// Setzen der Gitterpunkte des Definitionsbereich fr die Integration
		lengthx=main.riemannvisx.getPoints().length;
		lengthy=main.riemannvisy.getPoints().length;
		value = new double[lengthx - 1][lengthy - 1];
		int n=Math.max(lengthx,lengthy);
		main.gridpoints= new Punkt2D[2][n];
		main.gridpoints[0] = main.riemannvisx.getPoints();
		main.gridpoints[1] = main.riemannvisy.getPoints();
		grid = Grid2D.createGrid2D(main.gridpoints);
		this.vertices = new Vertex3D[5 * lengthx * lengthy];
		Vector vert = new Vector(lengthx * lengthy);
		Vector triangles = new Vector(8 * (lengthx - 1) * (lengthy - 1) + 2
				* ((lengthx - 1) + (lengthy - 1)));
		Random random = new Random();
		// Berechnen des Funktionswertes der Rechtecke
		// mittels s-stufiger zweidimensionaler Quadraturformel
		GaussIntegration2D integrator = new GaussIntegration2D();
		if (main.first == true) {
			double exactsum = integrator.quadrature(main.function, x.a, x.b,
					y.a, y.b, 8, 9);
			main.paneloutput.setExactValue(exactsum);
			main.first = false;
		}
		if (this.QuadraturFormel.isSelected()) {
			int m = 0;
			for (int i = 0; i < lengthx - 1; i++) {
				for (int j = 0; j < lengthy - 1; j++) {
					rechteck = new CoordinateRect2D(grid[i][j].x,
							grid[i + 1][j + 1].x, grid[i][j].y,
							grid[i + 1][j + 1].y);
					main.knotpoints = new double[lengthx * lengthy][2][s];
					main.weigths = new double[s][s];
					integrator.calcKnotsAndWeights(grid[i][j].x,
							grid[i + 1][j].x, grid[i][j].y, grid[i][j + 1].y,
							main.knotpoints[m], main.weigths, s);
					value[i][j] = integrator.quadrature(main.function,
							grid[i][j].x, grid[i + 1][j].x, grid[i][j].y,
							grid[i][j + 1].y, s, 1)
							/ (rechteck.getXRange() * rechteck.getYRange());
					sum += value[i][j]
							* (rechteck.getXRange() * rechteck.getYRange());
					m++;
				}
			}
			main.paneloutput.createOutput(sum, (lengthx - 1) * (lengthy - 1)
					* s, s);
		}
		if (main.panelpartialderivs.zufllig1.isSelected()) {
			int w = 0;
			main.knotpoints = new double[lengthx * lengthy][1][2];
			for (int i = 0; i < lengthx - 1; i++) {
				for (int j = 0; j < lengthy - 1; j++) {
					rechteck = new CoordinateRect2D(grid[i][j].x,
							grid[i + 1][j + 1].x, grid[i][j].y,
							grid[i + 1][j + 1].y);
					double zufallx = Math.abs(random.nextDouble()) % 1;
					double valuex = (-grid[i][j].x + grid[i + 1][j].x)
							* zufallx + grid[i][j].x;
					double zufally = Math.abs(random.nextDouble()) % 1;
					double valuey = (-grid[i][j].y + grid[i][j + 1].y)
							* zufally + grid[i][j].y;
					main.knotpoints[w][0][0] = zufallx;
					main.knotpoints[w][0][1] = zufally;
					value[i][j] = main.function.getValue(valuex, valuey);
					sum += value[i][j] * rechteck.getXRange()
							* rechteck.getYRange();
				}
			}
			main.paneloutput
					.createOutput(sum, (lengthx - 1) * (lengthy - 1), s);
		}
		if (main.panelpartialderivs.oberunter.isSelected()) {
			for (int i = 0; i < lengthx - 1; i++) {
				for (int j = 0; j < lengthy - 1; j++) {
					gridnewton = Grid2D.createGrid2D(grid[i][j].x,
							grid[i + 1][j + 1].x, grid[i][j].y,
							grid[i + 1][j + 1].y, (int) 20 / (lengthx - 1) + 1,
							(int) 20 / (lengthy - 1) + 1);
					rechteck = new CoordinateRect2D(grid[i][j].x,
							grid[i + 1][j + 1].x, grid[i][j].y,
							grid[i + 1][j + 1].y);
					rechteckit = rechteck.mult(1.05);
					valuedown = Double.NaN;
					valueup = Double.NaN;
					getmaxmin(i, j);
					sumup += valueup * rechteck.getXRange()
							* rechteck.getYRange();
					sumdown += valuedown
							* (rechteck.getXRange() * rechteck.getYRange());
					value[i][j] = main.panelpartialderivs.ober.isSelected() ? valueup
							: valuedown;
				}
			}
			main.paneloutput.createOutput(sumup, sumdown, (lengthx - 1)
					* (lengthy - 1));
		}
		int c = 0;

		// Erzeugt fr jeden Gitterpunkt die Knoten der Treppenfunktion
		// Je nachdem welche Ecke,Rand oder inner Punkt es ist
		for (int i = 0; i < lengthx; i++) {
			for (int j = 0; j < lengthy; j++) {
				if (i == 0 && j == 0) {
					vertices[c] = new Vertex3D(grid[i][j].x, grid[i][j].y, 0);
					vertices[c + 1] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 2] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 3] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i][j]);
				}
				if (i == 0 && j == lengthy - 1) {
					vertices[c] = new Vertex3D(grid[i][j].x, grid[i][j].y, 0);
					vertices[c + 1] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i][j - 1]);
					vertices[c + 2] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 3] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
				}
				if (i == lengthx - 1 && j == lengthy - 1) {
					vertices[c] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i - 1][j - 1]);
					vertices[c + 1] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 2] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 3] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
				}
				if (i == lengthx - 1 && j == 0) {
					vertices[c] = new Vertex3D(grid[i][j].x, grid[i][j].y, 0);
					vertices[c + 1] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 2] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i - 1][j]);
					vertices[c + 3] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
				}
				if (i == 0 && j != 0 && j != lengthy - 1) {
					vertices[c] = new Vertex3D(grid[i][j].x, grid[i][j].y, 0);
					vertices[c + 1] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i][j - 1]);
					vertices[c + 2] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 3] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i][j]);
				}
				if (i == lengthx - 1 && j != 0 && j != lengthy - 1) {
					vertices[c] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i - 1][j - 1]);
					vertices[c + 1] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 2] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i - 1][j]);
					vertices[c + 3] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
				}
				if (j == 0 && i != 0 && i != lengthx - 1) {
					vertices[c] = new Vertex3D(grid[i][j].x, grid[i][j].y, 0);
					vertices[c + 1] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 2] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i - 1][j]);
					vertices[c + 3] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i][j]);
				}
				if (j == lengthy - 1 && i != 0 && i != lengthx - 1) {
					vertices[c] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i - 1][j - 1]);
					vertices[c + 1] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i][j - 1]);
					vertices[c + 2] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
					vertices[c + 3] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							0);
				}
				if (j != 0 && j != lengthy - 1 && i != 0 && i != lengthx - 1) {
					vertices[c] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i - 1][j - 1]);
					vertices[c + 1] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i][j - 1]);
					vertices[c + 2] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i - 1][j]);
					vertices[c + 3] = new Vertex3D(grid[i][j].x, grid[i][j].y,
							value[i][j]);
				}
				vertices[c + 4] = new Vertex3D(grid[i][j].x, grid[i][j].y, 0);
				c = c + 5;
			}
		}

		for (int i = 0; i < lengthx - 1; i++) {
			for (int j = 0; j < lengthy - 1; j++) {
				// links unten
				int p1 = 5 * (lengthy) * (i + 1) + 5 * j;
				// rechts unten
				int p2 = 5 * (lengthy) * (i + 1) + 5 * (j + 1);
				// rechts oben
				int p3 = 5 * lengthy * i + 5 * (j + 1);
				// links oben
				int p4 = 5 * lengthy * i + 5 * j;

				// oben
				if (vertices[p4 + 2].mc.x3*vertices[p4 + 3].mc.x3>=0) {
					triangles.add(new Triangle(vertices[p3 + 1],
							vertices[p4 + 3], vertices[p3]));
					triangles.add(new Triangle(vertices[p4 + 2], vertices[p3],
							vertices[p4 + 3]));
				} else {
					triangles.add(new Triangle(vertices[p3 + 1],
							vertices[p4 + 3], vertices[p3 + 4]));
					triangles.add(new Triangle(vertices[p4 + 4],
							vertices[p3 + 4], vertices[p4 + 3]));
					triangles.add(new Triangle(vertices[p3 + 4],
							vertices[p4 + 4], vertices[p3]));
					triangles.add(new Triangle(vertices[p4 + 2], vertices[p3],
							vertices[p4 + 4]));
				}
				// links
				if (vertices[p4 + 1].mc.x3*vertices[p4 + 3].mc.x3>=0) {
					triangles.add(new Triangle(vertices[p4 + 3],
							vertices[p1 + 2], vertices[p4 + 1]));
					triangles.add(new Triangle(vertices[p1], vertices[p4 + 1],
							vertices[p1 + 2]));
				} else {
					triangles.add(new Triangle(vertices[p4 + 3],
							vertices[p1 + 2], vertices[p4 + 4]));
					triangles.add(new Triangle(vertices[p1 + 4],
							vertices[p4 + 4], vertices[p1 + 2]));
					triangles.add(new Triangle(vertices[p4 + 4],
							vertices[p1 + 4], vertices[p4 + 1]));
					triangles.add(new Triangle(vertices[p1], vertices[p4 + 1],
							vertices[p1 + 4]));
				}

				// Funktionsflche
				triangles.add(new Triangle(vertices[p4 + 3], vertices[p3 + 1],
						vertices[p1 + 2]));
				triangles.add(new Triangle(vertices[p2], vertices[p1 + 2],
						vertices[p3 + 1]));
				// Nullniveau
				triangles.add(new Triangle(vertices[p4 + 4], vertices[p3 + 4],
						vertices[p1 + 4]));
				triangles.add(new Triangle(vertices[p2 + 4], vertices[p1 + 4],
						vertices[p3 + 4]));
			}
		}
		for (int i = 0; i < lengthx - 1; i++) {
			// rechts unten
			int p2 = 5 * (lengthy) * (i + 2) - 5;
			// rechts oben
			int p3 = 5 * (lengthy) * (i + 1) - 5;
			// rechts
			if (vertices[p3 + 1].mc.x3*vertices[p3 + 3].mc.x3>=0) {
				triangles.add(new Triangle(vertices[p3 + 3], vertices[p2 + 2],
						vertices[p3 + 1]));
				triangles.add(new Triangle(vertices[p2], vertices[p3 + 1],
						vertices[p2 + 2]));
			} else {
				triangles.add(new Triangle(vertices[p3 + 3], vertices[p2 + 2],
						vertices[p3 + 4]));
				triangles.add(new Triangle(vertices[p2 + 4], vertices[p3 + 4],
						vertices[p2 + 2]));
				triangles.add(new Triangle(vertices[p3 + 4], vertices[p2 + 4],
						vertices[p3 + 1]));
				triangles.add(new Triangle(vertices[p2], vertices[p3 + 1],
						vertices[p2 + 4]));
			}
		}
		for (int i = 0; i < lengthy - 1; i++) {
			// links unten
			int p1 = 5 * (lengthy) * (lengthx - 1) + 5 * i;
			// rechts unten
			int p2 = 5 * (lengthy) * (lengthx - 1) + 5 * (i + 1);
			// unten
			if (vertices[p2].mc.x3*vertices[p2 + 1].mc.x3>=0) {
				triangles.add(new Triangle(vertices[p1 + 2], vertices[p2],
						vertices[p1 + 3]));
				triangles.add(new Triangle(vertices[p2 + 1], vertices[p1 + 3],
						vertices[p2]));
			} else {
				triangles.add(new Triangle(vertices[p1 + 2], vertices[p2],
						vertices[p1 + 4]));
				triangles.add(new Triangle(vertices[p2 + 4], vertices[p1 + 4],
						vertices[p2]));
				triangles.add(new Triangle(vertices[p1 + 4], vertices[p2 + 4],
						vertices[p1 + 3]));
				triangles.add(new Triangle(vertices[p2 + 1], vertices[p1 + 3],
						vertices[p2 + 4]));
			}
		}
		for (int i = 0; i < vertices.length; i++)
			vert.add(vertices[i]);
		Mesh mesh = new Mesh(triangles, vert);
		try {

			main.surface3d[1].setMesh(mesh);

		} catch (Exception e) {

			// Flche unsichtbar
			main.surface3d[1].setVisible(false);
			// Ausgabe der Fehlermeldung
			main.mathpanel3d.reportError(e);
			return;
		}
		// Neuzeichnen des Definitionsbereichs
		main.mathpanel2d.repaint();

		// aktuelle Einstellung des Panels der
		// Partitionierung auslesen und aktivieren
		setselected();

		// Grenzen der Flche
		CoordinateBox3D box = main.surface3d[0].getBoundingBox();
		box.combine(main.surface3d[1].getBoundingBox());

		// Grenzen setzen
		main.mathpanel3d.getScene3D().setBoundingBox(box);

		// Flche sichtbar
		main.surface3d[1].setVisible(true);

		// Meldung ber Erfolg und neu zeichnen
		main.mathpanel3d.reportSuccess(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.-1")
				+ Messages.getString(AppletRiemann3D.BUNDLE_NAME,
						"PanelSurface.1")); //$NON-NLS-1$
		main.mathpanel3d.repaint();
	}

	/** Ereignisverarbeitung */
	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("zufllig1")) {
			if (this.zufllig1.isSelected()) {
				main.panelpartialderivs.sliderStep.setEnabled(false);
				main.panelpartialderivs.sliderStep.setValue(1);
				main.panelpartialderivs.ober.setEnabled(false);
				main.panelpartialderivs.unter.setEnabled(false);
			} else {
				main.panelpartialderivs.sliderStep.setEnabled(true);
			}
		}
		if (event.getActionCommand().equals("ou")) {
			if (oberunter.isSelected()) {
				main.panelpartialderivs.sliderStep.setEnabled(false);
				main.panelpartialderivs.sliderStep.setValue(1);
				main.panelpartialderivs.ober.setEnabled(true);
				main.panelpartialderivs.unter.setEnabled(true);
			} else {
				main.panelpartialderivs.sliderStep.setEnabled(true);
			}
		}
		if (event.getActionCommand().equals("qf")) {
			if (this.QuadraturFormel.isSelected()) {
				main.panelpartialderivs.sliderStep.setEnabled(true);
				main.panelpartialderivs.sliderStep.setValue(1);
				main.panelpartialderivs.ober.setEnabled(false);
				main.panelpartialderivs.unter.setEnabled(false);
			} else {
				main.panelpartialderivs.sliderStep.setEnabled(false);
			}
		}
		if (event.getActionCommand().equals("zoom")) { //$NON-NLS-1$
			try {
				zoom();
			} catch (Exception e) {
				main.mathpanel2d.reportError(e);
			}
		}
		if (event.getActionCommand().equals("compute")) { //$NON-NLS-1$
			try {
				compute();
				zoom.setSelected(false);
				zoom();
			} catch (Exception e) {
				// Ausgabe der Fehlermeldung
				main.mathpanel3d.reportError(e);
				return;
			}
			main.mathpanel2d.repaint();
			main.surface3d[0].setVisible(false);
			main.surface3d[1].setVisible(true);

			// Flchenoptionen setzen
			main.visible.setEnabled(true);
			main.visible.radioswitch[1].setSelected(true);
			main.mathpanel3d.repaint();
		}
	}

	/** Initialisieren der Komponenten */
	void initComponents()  {

		// Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(1100, 600));

		// Optionsschaltflchen
		QuadraturFormel = new JRadioButton(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelPartialDerivates.4")); //$NON-NLS-1$
		QuadraturFormel.setActionCommand("qf"); //$NON-NLS-1$
		QuadraturFormel.addActionListener(this);
		QuadraturFormel.setToolTipText(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelPartialDerivates.6")); //$NON-NLS-1$

		zoom = new JRadioButton(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelPartialDerivates.18")); //$NON-NLS-1$
		zoom.setActionCommand("zoom"); //$NON-NLS-1$
		zoom.addActionListener(this);
		zoom.setToolTipText(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelPartialDerivates.20")); //$NON-NLS-1$

		zufllig1 = new JRadioButton(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelPartialDerivates.10")); //$NON-NLS-1$
		zufllig1.setActionCommand("zufllig1"); //$NON-NLS-1$
		zufllig1.addActionListener(this);
		zufllig1.setToolTipText(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelPartialDerivates.12")); //$NON-NLS-1$

		oberunter = new JRadioButton(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelPartialDerivates.22")); //$NON-NLS-1$
		oberunter.setActionCommand("ou"); //$NON-NLS-1$
		oberunter.addActionListener(this);
		oberunter.setToolTipText(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelPartialDerivates.24")); //$NON-NLS-1$

		ober = new JRadioButton(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelPartialDerivates.26")); //$NON-NLS-1$
		ober.setActionCommand("o"); //$NON-NLS-1$
		ober.addActionListener(this);
		ober.setToolTipText(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelPartialDerivates.28")); //$NON-NLS-1$
		ober.setEnabled(false);
		ober.setSelected(true);

		unter = new JRadioButton(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelPartialDerivates.30")); //$NON-NLS-1$
		unter.setActionCommand("u"); //$NON-NLS-1$
		unter.addActionListener(this);
		unter.setToolTipText(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelPartialDerivates.32")); //$NON-NLS-1$
		unter.setEnabled(false);

		QuadraturFormel.setSelected(true);

		ButtonGroup groupObenUnten = new ButtonGroup();
		groupObenUnten.add(ober);
		groupObenUnten.add(unter);

		ButtonGroup groupDerivateOpt = new ButtonGroup();
		groupDerivateOpt.add(QuadraturFormel);
		groupDerivateOpt.add(zufllig1);
		groupDerivateOpt.add(oberunter);

		// Panel zum Auswhlen an welchem Punkt die Ableit. gezeichnet wird
		main.panelchoose = new PanelChoose(main);

		// Label fr die Stufe der QF
		int n = 1;
		labelstep = new JLabel(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelSurface.5") + n); //$NON-NLS-1$ //$NON-NLS-2$
		labelstep.setFont(labelstep.getFont().deriveFont(Font.BOLD));

		// Label fr die Anzahl der Gitterpunkte
		int m = 5;
		labelgitter = new JLabel(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.4") + m + "x" + m); //$NON-NLS-1$ //$NON-NLS-2$
		labelgitter.setFont(labelgitter.getFont().deriveFont(Font.BOLD));

		// Slider fr Stufe der QF
		sliderStep = new JSlider();
		sliderStep.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent evt) {
				// aktuelle Gre im Label labelstep ausgeben
				JSlider slider = (JSlider) evt.getSource();
				int n = slider.getValue();
				labelstep
						.setText(Messages.getString(
								AppletRiemann3D.BUNDLE_NAME, "PanelSurface.23") + String.valueOf(n)); //$NON-NLS-1$
			}
		});
		// Wertebereich des Sliders einstellen
		sliderStep.setMinimum(1);
		sliderStep.setMaximum(8);
		sliderStep.setValue(1);

		// Slider fr Gitter
		slidergitter = new JSlider();
		slidergitter.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent evt){
				// aktuelle Gre im Label labelstep ausgeben
				JSlider slider = (JSlider) evt.getSource();
				int n = slider.getValue();
				labelgitter
						.setText(Messages.getString(
								AppletRiemann3D.BUNDLE_NAME, "PanelSurface.22") + String.valueOf(n) //$NON-NLS-1$
								+ "x" + String.valueOf(n)); //$NON-NLS-1$
				main.riemannvisx.setNumberSubDivisions(n-1);
				main.riemannvisy.setNumberSubDivisions(n-1);
				try {
					main.riemannvisx.create();
					main.riemannvisy.create();
				} catch (Exception e) {
					// TODO: handle exception
				}
				main.mathpanel2d.repaint();
			}
		});
		// Wertebereich des Sliders einstellen
		slidergitter.setMinimum(2);
		slidergitter.setMaximum(10);
		slidergitter.setValue(5);

		// Knopf "Zeichnen"
		buttonCalc = new JButton(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.26")); //$NON-NLS-1$
		buttonCalc.setActionCommand("compute"); //$NON-NLS-1$
		buttonCalc.addActionListener(this);
		buttonCalc.setToolTipText(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.27")); //$NON-NLS-1$

		// Komponenten zum Panel hinzufgen
		add(this.QuadraturFormel, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						0, 0, 0), 0, 0));

		add(labelstep, new GridBagConstraints(1, 0, 2, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 5, 10, 5), 0, 0));
		add(sliderStep, new GridBagConstraints(1, 1, 2, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(0, 5, 20, 5), 0, 0));

		add(oberunter, new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						0, 0, 0), 0, 0));

		add(ober, new GridBagConstraints(1, 2, 2, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 5, 10, 5), 0, 0));
		add(unter, new GridBagConstraints(1, 3, 2, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(0, 5, 20, 5), 0, 0));

		add(this.zufllig1, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						0, 0, 0), 0, 0));

		add(this.zoom, new GridBagConstraints(0, 4, 1, 1, 0.1, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						5, 0, 0, 0), 0, 0));

		add(labelgitter, new GridBagConstraints(1, 4, 2, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 5, 10, 5), 0, 0));

		add(slidergitter, new GridBagConstraints(1, 5, 2, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(0, 5, 20, 5), 0, 0));
		add(buttonCalc, new GridBagConstraints(0, 5, 2, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						20, 0, 0, 0), 0, 0));

		main.mathpanel2d.repaint();

	}

}