package uibk.applets.riemann3d;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import uibk.mtk.draw3d.objects.surface3d.Function2DMeshBuilder;
import uibk.mtk.draw3d.objects.surface3d.Mesh;
import uibk.mtk.geom.geom2d.CoordinateRect2D;
import uibk.mtk.geom.geom3d.CoordinateBox3D;
import uibk.mtk.lang.Messages;
import uibk.mtk.math.Interval;
import uibk.mtk.math.functions.ParsedFunction2D;
import uibk.mtk.math.parsing.Function2DParser;
import uibk.mtk.swing.base.IntervalTextField;
import uibk.mtk.swing.base.MPanel;
import uibk.mtk.swing.base.TextField;
import uibk.mtk.geom.geom2d.Punkt2D;

/**
 * Panel zum Definieren einer der Funktion, Auswahl der Gre und Form des
 * Gitters ber dem Flche visualisiert wird und des Bereiches des
 * Koordinatensystems ber dem die Funktion gezeichnet und die Riemann-Summe
 * gebildet wird.
 * 
 * <p>
 * Copyright: Institut fr Grundlagen der Bauingenieurwissenschaften, 2006
 * </p>
 * 
 * @author Andreas Rudigier (csae7164@uibk.ac.at)
 */
public class PanelSurface extends MPanel implements ActionListener {

	/** Textfeld fr die Formel der komplexen Funktion */
	TextField textFormula = new TextField();

	/** Button zum Starten des Plottens */
	JButton buttonCalc;

	/** Slider zur Anpassung der Gre des Gitters */
	JSlider sliderMeshPoints;

	/** Label mit der aktuellen Gre des Gitters */
	JLabel labelmesh;

	AppletRiemann3D main;

	/** Textfeld f?r den Bereich der x-Achse */
	IntervalTextField txtIntervalX;

	/** Textfeld f?r den Bereich der y-Achse */
	IntervalTextField txtIntervalY;

	// Wertebereich
	Interval intervalreal;

	Interval intervalimag;

	public PanelSurface(AppletRiemann3D main) {
		this.main = main;
		initComponents();
	}

	/**
	 * Wertet die im Feld <CODE>textFormula</CODE> eingebene Fkt. aus und
	 * speichert sie in <CODE>main.complexfunction</CODE>. Bei Fehler wir
	 * <CODE>false</CODE> zur?ckgeben und eine Fehlermelldung ausgegeben.
	 */
	private ParsedFunction2D getFunction() {

		try {
			String strfunc = textFormula.getText();
			return Function2DParser.parse(AppletRiemann3D.VARS[0],
					AppletRiemann3D.VARS[1], strfunc);

		} catch (Exception e) {
			main.mathpanel3d.reportError(Messages.getString(
					AppletRiemann3D.BUNDLE_NAME, "PanelSurface.0"), e); //$NON-NLS-1$

			return null;
		}

	}

	/** Berechnen und Plotten der Flche */
	void compute() {

		// Panel mit partielle Ableitungen deaktivieren
		main.panelpartialderivs.setEnabled(false);

		main.function = getFunction();
		// Parsen und setzen der Funktion
		if (main.function == null)
			return;

		// Gre des Gitters
		int n = sliderMeshPoints.getValue();

		// Berechnung des exakten Integrals zulassen
		main.first = true;

		try {
			intervalreal = txtIntervalX.getInterval();
			intervalimag = txtIntervalY.getInterval();
		} catch (Exception e) {

			main.mathpanel3d.reportError(e);
			return;
		}

		// Koordinatengrenzen
		CoordinateRect2D limits = new CoordinateRect2D(intervalreal.a,
				intervalreal.b, intervalimag.a, intervalimag.b);

		try {
			// Versuchen das Gitter fr die Triangulation zu berechnen
			Function2DMeshBuilder meshbuilder = new Function2DMeshBuilder(
					main.function);

			Mesh mesh = meshbuilder.build(limits, n, n, true);
			main.surface3d[0].setMesh(mesh);

			// Gitter zeichnen fr den Definitionsbereich der Integration
			main.panelpartialderivs.creategrid();

		} catch (Exception e) {

			// Flche unsichtbar
			main.surface3d[0].setVisible(false);
			// Ausgabe der Fehlermeldung
			main.mathpanel3d.reportError(e);
			return;
		}

		// Setzen der Gitterpunkte des Definitionsbereich fr die Integration
		main.gridpoints= new Punkt2D[2][5];
		main.gridpoints[0] = main.riemannvisx.getPoints();
		main.gridpoints[1] = main.riemannvisy.getPoints();
		
		// Grenzen der Flche
		CoordinateBox3D box = main.surface3d[0].getBoundingBox();

		// Grenzen setzen
		main.mathpanel3d.getScene3D().setBoundingBox(box);

		// Flchenoptionen setzen
		main.visible.setEnabled(false);
		
		// Flche sichtbar
		main.surface3d[0].setVisible(true);
		main.surface3d[1].setVisible(false);

		// Panel der Partitionierung aktivieren
		main.panelpartialderivs.setselected();

		// Meldung ber Erfolg und neu zeichnen
		main.mathpanel3d.reportSuccess(main.function
				+ Messages.getString(AppletRiemann3D.BUNDLE_NAME,
						"PanelSurface.1")); //$NON-NLS-1$
		main.mathpanel3d.repaint();
	}

	/** Ereignisverarbeitung: Berechnen der Flche */
	public void actionPerformed(ActionEvent ev) {
		if (ev.getActionCommand().equals(
				Messages.getString(AppletRiemann3D.BUNDLE_NAME,
						"PanelSurface.2"))) { //$NON-NLS-1$

			// Flche berechnen
			compute();
			// alles neu zeichnen
			main.mathpanel3d.repaint();
		}
	}

	/** Initialisieren der Komponenten */
	void initComponents() {

		// Rahmen und Titel
		TitledBorder border = BorderFactory.createTitledBorder(Messages
				.getString(AppletRiemann3D.BUNDLE_NAME, "PanelSurface.3")); //$NON-NLS-1$
		border.setTitleFont(border.getTitleFont().deriveFont(Font.BOLD));
		setBorder(border);

		// Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(2000, 270));
		setPreferredSize(new Dimension(2000, 270));

		// Label fr Anzahl Gitterpunkte
		int n = AppletRiemann3D.GRIDDEFAULT;
		labelmesh = new JLabel(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelSurface.4") + n + "x" + n); //$NON-NLS-1$ //$NON-NLS-2$
		labelmesh.setFont(labelmesh.getFont().deriveFont(Font.BOLD));

		// Textfelder
		String[] vars = AppletRiemann3D.VARS;

		textFormula = new TextField(10);
		textFormula.setActionCommand("compute"); //$NON-NLS-1$
		textFormula.addActionListener(this);
		textFormula
				.setToolTipText(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
						"PanelSurface.7") //$NON-NLS-1$
						+ vars[0]
						+ "," + vars[1] + Messages.getString(AppletRiemann3D.BUNDLE_NAME, "PanelSurface.9")); //$NON-NLS-1$ //$NON-NLS-2$

		txtIntervalY = new IntervalTextField(10, Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.10"), true); //$NON-NLS-1$
		txtIntervalY.setActionCommand(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.11")); //$NON-NLS-1$
		txtIntervalY.addActionListener(this);
		txtIntervalY.setToolTipText(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.12")); //$NON-NLS-1$

		txtIntervalX = new IntervalTextField(10, Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.13"), true); //$NON-NLS-1$
		txtIntervalX.setActionCommand("compute"); //$NON-NLS-1$
		txtIntervalX.addActionListener(this);
		txtIntervalX.setToolTipText(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.15")); //$NON-NLS-1$

		// Knopf "Zeichnen"
		buttonCalc = new JButton(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.16")); //$NON-NLS-1$
		buttonCalc.setActionCommand("compute"); //$NON-NLS-1$
		buttonCalc.addActionListener(this);
		buttonCalc.setToolTipText(Messages.getString(
				AppletRiemann3D.BUNDLE_NAME, "PanelSurface.18")); //$NON-NLS-1$

		// Beschfriftung fr Feld textFormula

		JLabel labelFormula = new JLabel("f(" + vars[0] + "," + vars[1] + ")="); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		// Slider fr Gitter
		sliderMeshPoints = new JSlider();
		sliderMeshPoints.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent evt) {
				// aktuelle Gre im Label labelmesh ausgeben
				JSlider slider = (JSlider) evt.getSource();
				int n = slider.getValue();
				labelmesh
						.setText(Messages.getString(
								AppletRiemann3D.BUNDLE_NAME, "PanelSurface.22") + String.valueOf(n) //$NON-NLS-1$
								+ "x" + String.valueOf(n)); //$NON-NLS-1$
			}
		});
		// Wertebereich des Sliders einstellen
		sliderMeshPoints.setMinimum(AppletRiemann3D.GRIDMIN);
		sliderMeshPoints.setMaximum(AppletRiemann3D.GRIDMAX);
		sliderMeshPoints.setValue(AppletRiemann3D.GRIDDEFAULT);

		// Komponenten zum Panel hinzufgen
		add(labelFormula, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 5, 5, 0), 0, 0));
		add(textFormula, new GridBagConstraints(1, 1, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 5, 5, 0), 0, 0));
		add(labelmesh, new GridBagConstraints(0, 2, 2, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 5, 10, 5), 0, 0));
		add(sliderMeshPoints, new GridBagConstraints(0, 3, 2, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(0, 5, 20, 5), 0, 0));
		add(new JLabel(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelSurface.24")), new GridBagConstraints(0, 4, 1, 1, 0.0, //$NON-NLS-1$
				0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,
				new Insets(0, 5, 5, 0), 0, 0));
		add(txtIntervalX, new GridBagConstraints(1, 4, 1, 1, 0.5, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 5, 5), 0, 0));
		add(new JLabel(Messages.getString(AppletRiemann3D.BUNDLE_NAME,
				"PanelSurface.25")), new GridBagConstraints(0, 5, 1, 1, 0.1, //$NON-NLS-1$
				0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,
				new Insets(0, 5, 5, 0), 0, 0));
		add(txtIntervalY, new GridBagConstraints(1, 5, 1, 1, 0.5, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 15, 5), 0, 0));
		add(buttonCalc, new GridBagConstraints(0, 8, 2, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						20, 0, 0, 0), 0, 0));

	}
}