package uibk.mtk.draw2d.objects;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Line2D;
import java.awt.image.BufferedImage;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Vector;

import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.draw2d.base.MathPanel2D;
import uibk.mtk.lang.Clickable;
import uibk.mtk.lang.Dragable;
import uibk.mtk.lang.PrepaintComputable;
import uibk.mtk.math.Interval;
import uibk.mtk.util.IntervalIterator;

/**
 * Visualisierung der Integration einer Funktion von R->R mit Riemannsummen
 * <p>
 * F?r eine Beschreibung des Applets siehe <a
 * href="http://www.mathe-online.at/nml/materialien/innsbruck/integration2d/">www.mathe-online.at/nml/materialien/innsbruck/integration2d/
 * </a>
 * </p>
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class RiemannSum2d extends Drawable2D implements PrepaintComputable,
		Dragable, Clickable {

	/** Farbe f?r den Rand der Rechtecke,.. */

	private Color edgecolor = Color.BLACK;

	private Color linescolor = Color.BLACK;

	/** Farbe der Punkte, die die Intervallgrenzen visualisieren */
	private Color pointcolor = Color.BLUE;

	/** Farbe f?r die Rechtecke,Trapeze,.. */
	private Color shapecolor = Color.BLACK;

	/** Ob x-Achse Unterteilt wird oder die y-Achse */
	boolean axes;

	boolean random = false;

	/** Radius der Punkte, die die Intervallgrenzen visualisieren */
	int pointradius = 3;

	/** aktuelle Summe der gew?hlten Quadratur */
	double sum;

	/** aktuell mit Maus verschobene Intervallgrenze */
	Dragable dragged = null;

	/** Formen der aktuellen Methode (z.B. Rechtecke, Trapeze) */
	Vector shapes = new Vector();

	/** Intervall auf dem die Unterteilung */
	Interval interval = new Interval(0, 1);

	/** Intervall wie hoch die Rechtecke sind */
	Interval intervalimag = new Interval(0, 1);

	/** Graph der Funktion */
	Graph2D graph = new Graph2D();

	/** Anzahl der Punkte mit denen der Graph gezeichnet wird */
	int numpointsgraph = 400;

	/** Vektor mit den Punkten die die Intervallgrenzen markieren */
	Vector points = new Vector();

	/** Anzahl der Unterteilungen */
	int n = 10;

	boolean negative_samecolor = true;

	/** Klasse f?r die Punkte die die Intervallgrenzen visualisieren */
	class IntervalPoint extends DragablePoint2D {

		/** Punkt links nebem diesem Punkt */
		IntervalPoint left = null;

		/** Punkt rechts nebem diesem Punkt */
		IntervalPoint right = null;

		public IntervalPoint(double x, double y) {
			super(x, y);
		}

		/** Ziehen des Punktes */
		public void drag(MouseEvent ev) {
			if (axes) {
				double t = scene2d.pixelToX(ev.getX());
				if (left == null && t < interval.a)
					return;
				if (right == null && t > interval.b)
					return;
				if (left != null && t <= left.x)
					return;
				if (right != null && t >= right.x)
					return;
				x = t;
			} else {
				double t = scene2d.pixelToY(ev.getY());
				if (left == null && t < interval.a)
					return;
				if (right == null && t > interval.b)
					return;
				if (left != null && t <= left.y)
					return;
				if (right != null && t >= right.y)
					return;
				y = t;
			}
		}
	}

	/** Erzeugt die Visualisierung */
	public RiemannSum2d() {
		graph.setColor(Color.blue);
		setColor(shapecolor);

	}

	/** Gibt zur?ck ob das Drag & Drop aktiviert ist. */
	public boolean isDragEnabled() {
		return true;
	}

	private void leftandright() {
		int k = points.size();
		if (k == 0)
			return;
		for (int i = 0; i < k; i++) {
			IntervalPoint p = (IntervalPoint) points.elementAt(i);
			if (i < k - 1)
				p.right = (IntervalPoint) points.elementAt(i + 1);
			if (i > 0)
				p.left = (IntervalPoint) points.elementAt(i - 1);
		}
		IntervalPoint p = (IntervalPoint) points.elementAt(k - 1);
		p.right = null;
	}

	/** Hinzuf?gen einer neuen Intervallgrenze */
	private void addNewIntervallPointX(double x) {

		// wir haben mindestens zwei Punkte
		double min = ((IntervalPoint) points.elementAt(0)).getX();
		double max = ((IntervalPoint) points.elementAt(points.size() - 1))
				.getX();

		// neuer Punkt ist ganz links
		if (x < min)
			return;

		// neuer Punkt ist ganz rechts
		if (x > max)
			return;

		// neuer Punkt ist irgendwo in der Mitte->an der richtigen Stelle
		// hinzuf?gen
		ListIterator it = points.listIterator();
		IntervalPoint left = null;
		int i = 0;
		while (it.hasNext()) {
			IntervalPoint p = (IntervalPoint) it.next();

			if (left != null && x > left.getX() && x < p.getX()) {

				IntervalPoint newpoint = createpoint(x);
				points.add(i, newpoint);

				return;
			}

			i++;

			left = p;
		}

	}

	/** Hinzuf?gen einer neuen Intervallgrenze */
	private void addNewIntervallPointY(double x) {

		// wir haben mindestens zwei Punkte
		double min = ((IntervalPoint) points.elementAt(0)).getY();
		double max = ((IntervalPoint) points.elementAt(points.size() - 1))
				.getY();

		// neuer Punkt ist ganz links
		if (x < min)
			return;
		// neuer Punkt ist ganz rechts
		if (x > max)
			return;

		// neuer Punkt ist irgendwo in der Mitte->an der richtigen Stelle
		// hinzuf?gen
		ListIterator it = points.listIterator();
		IntervalPoint left = null;
		int i = 0;
		while (it.hasNext()) {
			IntervalPoint p = (IntervalPoint) it.next();
			if (left != null && x > left.getY() && x < p.getY()) {
				IntervalPoint newpoint = createpoint(x);
				points.add(i, newpoint);
				return;
			}
			i++;
			left = p;
		}
	}

	/** Implementierung des <CODE>Clickable</CODE> -Interfaces */
	public boolean clickcontains(int x, int y) {

		double xx = scene2d.pixelToX(x);
		double yy = scene2d.pixelToY(y);
		if (axes) {
			if (xx < interval.a || xx > interval.b)
				return false;

			if (yy < intervalimag.a || yy > intervalimag.b)
				return false;
		} else {
			if (yy < interval.a || yy > interval.b)
				return false;

			if (xx < intervalimag.a || xx > intervalimag.b)
				return false;
		}
		return true;

	}

	/**
	 * Implementierung des <CODE>Clickable</CODE> -Interfaces, wird aufgerufen
	 * falls im Mathpanel geclickt wird.
	 */
	public void click(MouseEvent ev) {

		double x = scene2d.pixelToX(ev.getX());
		double y = scene2d.pixelToY(ev.getY());
		// Rechte Maustaste ->Hinzuf?gen eines neuen Punktes
		if (onlyRightButton(ev)) {
			if (axes) {
				if (y < (interval.a + intervalimag.getLength() /30)
						&& y >(interval.a - intervalimag.getLength()/30)) {
					addNewIntervallPointX(x);
				} 
			} else {

				if (x < (interval.a + intervalimag.getLength() /30)
						&& x >(interval.a - intervalimag.getLength()/30)) {
					addNewIntervallPointY(y);
				}
			}
		}

		// Mittlere Maustaste : Entfernen eines Punktes
		if (onlyMiddleButton(ev)) {
			if (points.size() < 2)
				return;
			ListIterator it = points.listIterator();
			int i = 0;
			while (it.hasNext()) {
				IntervalPoint p = (IntervalPoint) it.next();
				if (p.dragcontains(ev.getX(), ev.getY())) {
					// if (i!=0&&i!=points.size()-1)
					points.remove(i);
					break;
				}
				i++;
			}
		}

		// neu sortieren
		leftandright();

		// zeichnen
		panel.repaint();
	}

	/**
	 * Gibt zur?ck ob beim MouseEvent <CODE>ev</CODE> nur der linke Knopf
	 * gedr?ckt worden sit
	 */
	private boolean onlyLeftButton(MouseEvent ev) {
		return (ev.getModifiers() == InputEvent.BUTTON1_MASK);
	}

	/**
	 * Gibt zur?ck ob beim MouseEvent <CODE>ev</CODE> nur der rechte Knopf
	 * gedr?ckt worden sit
	 */
	private boolean onlyRightButton(MouseEvent ev) {
		return (ev.getModifiers() == InputEvent.BUTTON3_MASK);
	}

	/**
	 * Gibt zur?ck ob beim MouseEvent <CODE>ev</CODE> nur der mittlere Knopf
	 * gedr?ckt worden sit
	 */
	private boolean onlyMiddleButton(MouseEvent ev) {
		return (ev.getModifiers() == InputEvent.BUTTON2_MASK);
	}

	/** Gibt zur?ck ob das Ver?ndern der Intervallgrenzen aktiviert ist */
	public boolean randomEnabled() {
		return this.random;
	}

	private double[] rand() {
		int k = n + 1;

		double xmin = interval.a;
		double xmax = interval.b;
		double delta = interval.getLength() / n;
		double minlength = delta / 2;
		double maxlength = delta * 1.5;

		Double[] numbers = new Double[n];

		double summe = 0;

		for (int i = 0; i < n / 2; i++) {
			double t = rand(minlength, maxlength);
			numbers[i] = new Double(t);
			summe += t;
		}

		double dt = (interval.getLength() - summe) / (n / 2 + 1);

		for (int i = n / 2; i < n; i++) {

			numbers[i] = new Double(dt);

		}
		Collections.shuffle(Arrays.asList(numbers));

		double[] limits = new double[k];
		double pos = xmin;
		limits[0] = xmin;
		limits[k - 1] = xmax;
		for (int i = 1; i < k - 1; i++) {
			pos += numbers[i].doubleValue();
			limits[i] = pos;
		}

		return limits;
	}

	private static double rand(double xmin, double xmax) {
		return xmin + Math.random() * (xmax - xmin);
	}

	public void randomSubDivisions(boolean b) {
		random = b;
	}

	/** Ziehen des ausgew?hlten Punktes */
	public void drag(MouseEvent ev) {
		if (!onlyLeftButton(ev))
			return;
		if (dragged == points.get(0))
			return;
		if (dragged == points.get(points.size()-1))
			return;
		if (axes) {
			dragged.drag(ev);
		} else {
			dragged.drag(ev);
		}
	}

	/**
	 * Gibt zur?ck, ob die Maus im "Ziehbereich" eines der Punkte ist
	 */
	public boolean dragcontains(int xm, int ym) {
		Iterator it = points.iterator();
		while (it.hasNext()) {

			DragablePoint2D mathpoint = (DragablePoint2D) it.next();
			if (mathpoint.dragcontains(xm, ym)) {
				dragged = mathpoint;
				return true;
			}

		}
		return false;
	}

	/** Ziehen aktivieren/deaktivieren */
	public void enabledrag(boolean enable) {

	}

	/** Funktion setzen die integriert werden soll */
	public void setIntervalimag(Interval inter) {
		this.intervalimag = (Interval) inter.clone();

	}

	/** Setzt das Intervall fest ?ber dem integriert werden soll */
	public void setAxes(boolean is) {
		this.axes = is;

	}

	/** Setzt das Intervall fest ?ber dem integriert werden soll */
	public void setInterval(Interval inter) {
		this.interval = (Interval) inter.clone();

	}

	/** Setzt die Anzahl der Unterteilungen des Integrationsintervalls */
	public void setNumberSubDivisions(int size) {
		if (size != n) {
			this.n = size;
		}

	}

	/**
	 * Erzeugt die Punkte an den Intervallgrenzen x-Achse(die Unterteilung ist
	 * ?quidistant)
	 * 
	 */
	private void createIntervalPointsX() {

		points.clear();

		if (random) {

			double[] limits = rand();
			for (int i = 0; i < limits.length; i++) {
				IntervalPoint p = createpoint(limits[i]);
				points.add(p);

			}

		} else {

			IntervalIterator it = new IntervalIterator(interval, n + 1);
			while (it.hasNext()) {

				IntervalPoint p = createpoint(it.nextdouble());
				points.add(p);

			}

		}
		leftandright();
	}

	/**
	 * Erzeugt die Punkte an den Intervallgrenzen y-Achse (die Unterteilung ist
	 * ?quidistant)
	 * 
	 */
	private void createIntervalPointsY() {

		points.clear();

		if (random) {

			double[] limits = rand();
			for (int i = 0; i < limits.length; i++) {
				IntervalPoint p = createpoint(limits[i]);
				points.add(p);

			}

		} else {

			IntervalIterator it = new IntervalIterator(interval, n + 1);
			while (it.hasNext()) {

				IntervalPoint p = createpoint(it.nextdouble());
				points.add(p);

			}

		}
		leftandright();
	}

	/** Erzeugt einen Intervall-Endpunkt an der Stelle</CODE> x</CODE> */
	IntervalPoint createpoint(double x) {
		IntervalPoint p;
		if (axes) {
			p = new IntervalPoint(x, intervalimag.a);
		} else {
			p = new IntervalPoint(intervalimag.a, x);
		}

		p.setColor(pointcolor);
		p.setRadius(pointradius);
		p.setMathPanel2D(panel);
		return p;
	}

	/** Setzen des Mathpanel */
	public void setMathPanel2D(MathPanel2D panel) {
		super.setMathPanel2D(panel);
	}

	/** Berechnung der Komponenten der Visualiserung */
	public void prepaintcompute() throws Exception {
		computeshapes();
	}

	private void computeshapes() {
		shapes.clear();

		lines.clear();

		int yPix0 = scene2d.yToPixel(intervalimag.a);
		int xPix0 = scene2d.xToPixel(intervalimag.a);
		IntervalPoint left = null;
		IntervalPoint right = null;

		Iterator it = points.iterator();

		while (it.hasNext()) {

			IntervalPoint p = (IntervalPoint) it.next();
			right = p;

			if (left != null)
				if (axes) {

					double xl = left.getX();
					double xr = right.getX();

					int xPixLeft = scene2d.xToPixel(xl);
					int xPixRight = scene2d.xToPixel(xr);
					int yPixFMid = scene2d.yToPixel(intervalimag.b);

					MyPolygon poly = new MyPolygon();

					poly.addPoint(xPixLeft, yPix0);
					poly.addPoint(xPixLeft, yPixFMid);
					poly.addPoint(xPixRight, yPixFMid);
					poly.addPoint(xPixRight, yPix0);
					shapes.add(poly);
				} else {

					double xl = left.getY();
					double xr = right.getY();

					int xPixLeft = scene2d.yToPixel(xl);
					int xPixRight = scene2d.yToPixel(xr);
					int yPixFMid = scene2d.xToPixel(intervalimag.b);

					MyPolygon poly = new MyPolygon();

					poly.addPoint(xPix0, xPixLeft);
					poly.addPoint(yPixFMid, xPixLeft);
					poly.addPoint(yPixFMid, xPixRight);
					poly.addPoint(xPix0, xPixRight);
					shapes.add(poly);
				}
			left = right;
		}
	}

	Vector lines = new Vector();

	/** Zeichnen der Visualisierung */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {

		Iterator it = shapes.iterator();

		// Zeichnen der Rechtecke,..

		while (it.hasNext()) {
			MyPolygon shape = (MyPolygon) it.next();
			g2.setColor(edgecolor);
			g2.draw(shape);
		}

		it = lines.iterator();

		// Zeichnen der Rechtecke,..
		g2.setColor(linescolor);
		while (it.hasNext()) {
			g2.draw((Line2D.Double) it.next());
		}

		// falls Intervallpunkte aktiviert ->Punkte zeichnen
		it = points.iterator();
		while (it.hasNext()) {

			DragablePoint2D mathpoint = (DragablePoint2D) it.next();
			mathpoint.draw(framebuffer, g2);
		}

	}

	/** Erzeugt die Visualisierung mit der aktuellen Einstellungen */
	public void create() throws Exception {
		if (axes) {
			createIntervalPointsX();
		} else {
			createIntervalPointsY();
		}
		computeshapes();

	}
	/** Gibt die Anzahl der Unterteilungen zur?ck */
	public int getNumberSubDivisions() {
		return n;
	}

	public Interval getInterval() {

		return (Interval) interval.clone();
	}
	
	/** Gibt die Intervalendpunkte zurck */
	public Punkt2D[] getPoints() {
		Punkt2D[] point=new Punkt2D[points.size()];
		for (int i = 0; i < point.length; i++) {
			IntervalPoint p=(IntervalPoint) this.points.elementAt(i);
			point[i]=p.getPunkt2D();
		}
		return point;
	}

	class MyPolygon extends Polygon {

		Color fillcolor;
	}

	public void enableSameColor(boolean b) {
		this.negative_samecolor = b;
	}

	public boolean isEnabledSameColor() {
		return negative_samecolor;
	}

}