package uibk.mtk.draw3d.axescube3d;

import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;

import uibk.mtk.draw3d.base.Scene3D;
import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.geom.geom2d.Vector2D;
import uibk.mtk.geom.geom3d.Punkt3D;
import uibk.mtk.geom.geom3d.Vector3D;

/**
 * Ein "Tick" auf einer Achse
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class Tick {

	/** Startpunkt des Tick in Modellkoordinaten */
	Punkt3D mc_pos;

	/** Richtung des Tick in Modellkoordinaten */
	Vector3D mc_dir;

	/** Die Achsenbox zu der der Tick geh?rt */
	AxesCube3D axes;

	/** gro?er Tick ja/nein */
	boolean big = false;

	/** Startpunkt des Tick in Ger?tekoordinaten */
	Point start = new Point();

	/** Endpunkt des Tick in Ger?tekoordinaten */
	Point end = new Point();

	/** Position der Beschriftung in Ger?tekoordinaten */
	Point labelpos = new Point();

	/** Beschriftung des Tick */
	String label;

	/** wird die Beschriftung gezeichnet */
	private boolean drawlabel;

	/**
	 * Erzeugt den Tick
	 * 
	 * @param axes
	 *            Achsenw?rfel
	 * @param posmc
	 *            Position des Tick in Modellkoordinaten
	 * @param dirmc
	 *            Richtung des Tick in Modellkoordinaten
	 * @param big
	 *            gro?er Tick ja/nein
	 * @param label
	 *            Beschriftung des Tick
	 */
	public Tick(AxesCube3D axes, Punkt3D posmc, Vector3D dirmc, boolean big,
			String label) {

		this.mc_pos = posmc;
		this.mc_dir = dirmc;
		this.big = big;
		this.axes = axes;
		this.label = label;

	}

	/**
	 * ?berpr?ft ob die Beschriftung des Tick sich mit den anderen
	 * Beschriftungen der Achsenbox ?berlappt, falls ja wird die Var. <CODE>
	 * drawlabel</CODE> auf <CODE>false</CODE> gesetzt
	 * 
	 * @param g2
	 */
	private void checkIntersection(Graphics g2) {

		FontMetrics metrics = g2.getFontMetrics();
		int height = metrics.getAscent();
		int length = metrics.stringWidth(label);

		Rectangle r = new Rectangle(labelpos.x, labelpos.y - height, length,
				height);
		axes.labelarea.add(r);
		for (int i = 0; i < axes.labelarea.size() - 1; i++) {
			Rectangle2D rect = (Rectangle2D) axes.labelarea.elementAt(i);

			if (r.intersects(rect)) {
				drawlabel = false;
				break;
			}
		}

	}

	/** Zeichnen des Ticks und gegebenenfalls seiner BEschriftung */
	public void draw(Graphics2D g2) {

		g2.setColor(axes.getColor());

		if (big && drawlabel) {
			//Beschriftung zeichnen
			g2.drawString(label, labelpos.x, labelpos.y);
		}

		g2.drawLine(start.x, start.y, end.x, end.y);
	}

	/**
	 * Berechnen des Ticks, d.h seiner Position und falls es ein gro?er Tick ist
	 * der Position seiner Beschriftung
	 */
	public void compute(Graphics2D g2, Scene3D scene3d) {
		drawlabel = true;

		//Startpunkt Ger?tekoordinaten
		Punkt2D startp = scene3d.project(mc_pos);
		start.x = (int) startp.x;
		start.y = (int) startp.y;

		//Richtung des Tick in Ger?tekoordinaten
		Vector2D tickdir = scene3d.projectVector(mc_pos, mc_dir);
		double scale = big ? AxesCube3D.BIGTICKLENGTH
				: AxesCube3D.SMALLTICKLENGTH;

		//Endpunkt in Ger?tekoordinaten
		Punkt2D endp = startp.add(tickdir.scale(scale));
		end.x = (int) endp.x;
		end.y = (int) endp.y;

		if (big) {//falls gro? -> Position der Beschriftung

			//Breite und H?he der Beschriftung
			FontMetrics metrics = g2.getFontMetrics();
			int height = metrics.getHeight();
			int length = metrics.stringWidth(label);

			//Richtung des Tick bestimmen
			tickdir.normalize();
			byte direction = AxesCube3D.getDirection(tickdir);

			//Position der Beschriftung bestimmen
			double offsetx = 0;
			double offsety = 0;

			if ((AxesCube3D.TOP & direction) > 0) {
				offsety = AxesCube3D.BIGTICKLENGTH + AxesCube3D.GAPTOLABEL
						+ height;
			} else {
				offsety = AxesCube3D.BIGTICKLENGTH + AxesCube3D.GAPTOLABEL;

			}
			if ((AxesCube3D.RIGHT & direction) > 0) {
				offsetx = (AxesCube3D.BIGTICKLENGTH + AxesCube3D.GAPTOLABEL);
			} else {
				offsetx = AxesCube3D.BIGTICKLENGTH + AxesCube3D.GAPTOLABEL
						+ length;

			}
			tickdir.x *= offsetx;
			tickdir.y *= offsety;

			labelpos.x = (int) (startp.x + tickdir.x);
			labelpos.y = (int) (startp.y + tickdir.y);

			//auf ?berlappungen ?berpr?fen
			checkIntersection(g2);

		}
	}

}