package uibk.mtk.draw3d.base;

import uibk.mtk.lang.Drawable;

/**
 * Ein Drawable3D Objekt ist ein Objekt im Raum das gezeichnet werden kann. Es
 * kann zu einem MathPanel3d hinzugef?gt werden welches wiederum eine <CODE>
 * Scene3D</CODE> enth?lt, in dem sich die Drawable Objekte "befinden". Falls
 * ein Drawable Objekt aufwendigere Berechnungen durchf?hren muss, sollte es das
 * Interface <CODE>PrepaintComputable</CODE> implementieren. (Die Funktion
 * prepaintcompute() des Interfaces wird vom dem <CODE>Mathpanel3d</CODE>, in
 * dem sich das Drawable befindet vor dem Zeichnen aufgerufen (falls sich
 * irgendetwas ge?ndert hat, wie z.B. die Fenstergr??e)
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public abstract class Drawable3D extends Drawable {

	/**
	 * die 3D-Szene, in dem sich das Drawable befindet, wird von der Funktion
	 * setMathPanel3d auf das zum MathPanel3d geh?rende Koordinatensystem
	 * gesetzt
	 */
	protected Scene3D scene3d = null;

	/** Das verwaltende Mathpanel3d Objekt in dem sich das Drawable befindet */
	protected MathPanel3D panel = null;

	/**
	 * Setzt die Membervariable panel, also das Panel Objekt in dem das
	 * Drawable-Objekt gezeichnet wird. Wir f?hren die Membervarialbe scene3d in
	 * den Drawables extra noch mal ein, da wir die Scene3D oft brauchen und
	 * Zugriff ?ber panel etwas m?hsamer w?re.
	 */
	public void setMathPanel3D(MathPanel3D panel) {
		this.panel = panel;
		this.scene3d = panel.getScene3D();
	}

	public void repaint() {
		panel.repaint();
	}
}