package uibk.mtk.draw3d.objects.surface3d;

import java.awt.Color;
import java.util.Iterator;

import uibk.mtk.geom.geom3d.CoordinateBox3D;
import uibk.mtk.math.MathUtil;

/**
 * Ein Farbmodell das die Vertice nach ihrer z-Koordinate f?rbt.
 * 
 * <p>
 * Die Farbe f?r einen Vertex wird durch lineare Interpolation nach der
 * z-Koordinate bestimmt wobei f?r die minimale und maximale z-Koordinate eine
 * Farbe gegeben ist bzw. gesetzt werden kann.
 * </p>
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class ColorModelZ extends ColorModel {

	/** Farbe bei minimaler z-Koordinate */
	private Color colormin = new Color(0, 0, 220);

	/** Farbe bei maximaler z-Koordinate */
	private Color colormax = new Color(220, 10, 20);

	public void colorVertices(Iterator it, CoordinateBox3D cube) {

		int red_min = colormin.getRed();
		int red_max = colormax.getRed();
		int blue_min = colormin.getBlue();
		int blue_max = colormax.getBlue();
		int green_min = colormin.getGreen();
		int green_max = colormax.getGreen();

		while (it.hasNext()) {

			Vertex3D vertex = (Vertex3D) it.next();

			//Interpolation des Rot/Gr?n und Blauanteils
			int r = (int) MathUtil.interpolate(cube.zmin, cube.zmax, red_min,
					red_max, vertex.mc.x3);
			int g = (int) MathUtil.interpolate(cube.zmin, cube.zmax, green_min,
					green_max, vertex.mc.x3);
			int b = (int) MathUtil.interpolate(cube.zmin, cube.zmax, blue_min,
					blue_max, vertex.mc.x3);

			vertex.color = new Color(r, g, b);
		}
	}

	/** Setzt die Farbe bei minimaler z-Koordinate */
	public void setColorMin(Color min) {

		colormin = new Color(min.getRGB());
	}

	/** Setzt die Farbe bei maximaler z-Koordinate */
	public void setColorMax(Color max) {

		colormax = new Color(max.getRGB());
	}

}