package uibk.mtk.draw3d.rasterizer;

import java.awt.Color;

/**
 * Hilfsklasse f?r RGB-Farben mit einfacher Genauigkeit(wird verwendet von
 * Klasse Surface3D und Rasterizer)
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public final class Farbe {

	/** Rotanteil der Farbe ([0,255]) */
	float r;

	/** Gr?nanteil der Farbe ([0,255]) */
	float g;

	/** Blauanteil der Farbe ([0,255]) */
	float b;

	/** die Farbe schwarz */
	public static final Farbe BLACK = new Farbe(0, 0, 0);

	/** die Farbe wei? */
	public static final Farbe WHITE = new Farbe(255, 255, 255);

	/** Erzeugt eine Farbe mit den angegebenen Farbanteilen */
	public Farbe(float r, float g, float b) {
		this.r = r;
		this.g = g;
		this.b = b;
	}

	/** Erzeugt die Farbe schwarz */
	public Farbe() {
	}

	/** Erzeugt eine Farbe mit den Farbanteilen der ?bergenenen Farbe */
	public Farbe(Farbe f) {
		r = f.r;
		g = f.g;
		b = f.b;
	}

	/** Erzeugt eine Farbe mit den Farbanteilen der ?bergenenen Color */
	public Farbe(Color color) {
		r = color.getRed();
		g = color.getGreen();
		b = color.getBlue();
	}

	/** Addition zweier Farben */
	public void addself(Farbe c) {
		r = r + c.r;
		g = g + c.g;
		b = b + c.b;
	}

	/** Subtraktion zweier Farben */
	public void subself(Farbe c) {
		r = r - c.r;
		g = g - c.g;
		b = b - c.b;
	}

	/** Gibt einen String der Form (r,g,b) zur?ck */
	public String toString() {
		return "(" + r + "," + g + "," + b + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	}

	/** Skalierung der Farbe c1 mit s */
	public Farbe scale(float s) {
		float nr = s * r;
		float ng = s * g;
		float nb = s * b;
		return new Farbe(nr, ng, nb);
	}

	/** Skalierung der Farbe c1 mit s */
	public Farbe add(Farbe f) {

		return new Farbe(f.r + r, f.g + g, f.b + b);
	}

	/** Skalierung der Farbe c1 mit s */
	public Farbe sub(Farbe f) {

		return new Farbe(r - f.r, g - f.g, b - f.b);
	}

	/** Skalierung der Farbe mit s */
	public void scaleself(float s) {
		r = s * r;
		g = s * g;
		b = s * b;
	}

	/** setzt die Farbanteile auf die Farbanteile von c */
	public void set(Farbe c) {
		r = c.r;
		g = c.g;
		b = c.b;
	}

	/** setzt die Farbanteile auf die Farbanteile von c */
	public void set(Color c) {
		r = c.getRed();
		g = c.getGreen();
		b = c.getBlue();
	}

	/**
	 * Lineare Interpolation
	 * 
	 * @param a
	 *            "linker" Wert
	 * @param b
	 *            "rechter" Wert
	 * @param colora
	 *            Farbe bei a
	 * @param colorb
	 *            Farbe bei b
	 * @param x
	 *            Wert an dem Farbe interpoliert werden soll (ein Wert zwischen
	 *            aund b)
	 */
	public static Farbe interpolate(float a, float b, Farbe colora,
			Farbe colorb, double x) {
		if (b - a == 0)
			return colora;
		else {
			float k = (float) (b - x) / (b - a);
			float l = (float) (x - a) / (b - a);
			return (colora.scale(k)).add(colorb.scale(l));
		}
	}

	/** Gibt den RGB-Wert der Farbe zur?ck */
	public int toInteger() {
		return (int) b + ((int) g << 8) + ((int) r << 16);
	}

	/** Konvertiert die Farbe zu einer <CODE>Color</CODE> */
	public Color toColor() {
		return new Color((int) r, (int) g, (int) b);
	}
}