package uibk.mtk.lang;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

/**
 * Ein Drawable Objekt ist ein Objekt, das gezeichnet werden kann.
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public abstract class Drawable {

	/**
	 * Funktion zum Zeichnen des Drawable
	 * 
	 * @param framebuffer
	 *            aus dem Graphikkontext ertelltes BufferedImage
	 * @param g2
	 *            Graphikkontext
	 */
	public abstract void draw(BufferedImage framebuffer, Graphics2D g2);

	/**
	 * Die Zeichenfarbe des Drawable Objekts, alle anderen Graphikeinstellungen
	 * (Stroke, ...) sollten nach Fertigstellen des Zeichnens (in der Funktion
	 * draw) wieder auf die ursprungliche Einstellung zur?ckgesetzt werden
	 */
	protected Color color = Color.black;

	/**
	 * gibt an, ob das Drawable gezeichnet werden soll
	 */
	protected boolean visible = true;

	/**
	 * kann von einem Drawable verwendet werden wenn es nicht gezeichnet werden
	 * kann (z.B. Berechnungen in PrepaintCompute ergeben, dass es nicht
	 * gezeichnet werden kann) und man nicht die Variable visible verwenden
	 * will,da diese von aussen gesetzt wird
	 */
	protected boolean draw = true;

	/**
	 * Gibt <CODE>true</CODE> zur?ck falls das Drawable sichtbar ist. Nur ein
	 * sichtbares Objekt wird von dem <CODE>MathPanel</CODE> in dem es sich
	 * befindet gezeichnet.
	 */
	public boolean isVisible() {
		return visible;
	}

	/**
	 * Setzt die zum Zeichnen verwendende Farbe fest
	 * 
	 * @param color
	 *            zu verwendende Farbe
	 */
	public void setColor(Color color) {
		this.color = color;
	}

	/**
	 * Gibt die Farbe des <CODE>Drawable</CODE> zur?ck
	 */
	public Color getColor() {
		return color;
	}

	/**
	 * Die Sichtbarkeit des Drawable Objektes setzen.
	 */
	public void setVisible(boolean v) {

		visible = v;

	}

}