package uibk.mtk.lang;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.UIManager;

/**
 * Klasse die einige globale Optionen f?r die Applets festlegt (Farben f?r
 * Komponenten, Hintergrundfarbe, LookAndFeel)
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public final class StandardOptions extends Options {

	public StandardOptions() {
	}

	/** Farbe f?r die Swing-Komponenten der Applets */
	private static final Color COMPONENTSCOLOR = new Color(230, 230, 230);

	/** Hintergrundfarbe f?r die Applets */
	private static final Color BACKGROUNDCOLOR = new Color(241, 241, 229);

	public Color getBackgroundColor() {
		return BACKGROUNDCOLOR;
	}

	public Color getComponentsColor() {
		return COMPONENTSCOLOR;
	}

	/**
	 * Funktion zum setzen der Hinweise zum Rendern (Antialiasing wird
	 * eingeschalten)
	 */
	public void setRenderingHints(Graphics2D g2) {
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setRenderingHint(RenderingHints.KEY_RENDERING,
				RenderingHints.VALUE_RENDER_DEFAULT);
	}

	/** setzt das LookAndFeel auf das SystemLookAndFeel */
	public void setLookAndFeel() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			changesLookAndFeel();

		} catch (Exception e) {
		}
	}

	/**
	 * Kleine ?nderungen (Farbe der Komponenten) am LookAndFeel vornehmen falls
	 * das Windows Look-and-Feel verwendet wird
	 */
	private void changesLookAndFeel() {
		if (UIManager.getLookAndFeel().getID().equals("Windows")) { //$NON-NLS-1$
			UIManager.put("Panel.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("Button.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("RadioButton.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("TabbedPane.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("ToolBar.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("Label.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("Slider.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("Label.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("CheckBox.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("CheckBox.background", COMPONENTSCOLOR); //$NON-NLS-1$
			UIManager.put("TableHeader.background", COMPONENTSCOLOR); //$NON-NLS-1$

		}

		//UIManager.put("ComboBox.selectionBackground",HIGHLIGHTCOLOR);
		//  UIManager.put("ComboBox.selectionForeground",Color.black);

	}

}