package uibk.mtk.math.functions;

import uibk.mtk.lang.Messages;
import uibk.mtk.parser.realparser.Expr;
import uibk.mtk.parser.realparser.Variable;

/**
 * Klasse f?r eine reelle Funktion von R^n->R^m die durch Benutzereingaben
 * bestimmt wird.
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class ParsedFunction extends Function {

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.math.functions.messages";//$NON-NLS-1$

	/** die Variablen der Funktion */
	Variable[] variables;

	/** die Komponenten der Funktion */
	Expr[] functions;

	/** Erzeugt die Funktion */
	public ParsedFunction(Variable[] variables, Expr[] functions) {
		super(variables.length, functions.length);
		this.functions = functions;

		this.variables = variables;

	}

	/**
	 * Gibt den Wert der Funktion an der Stelle <CODE>x</CODE> zur?ck
	 * 
	 * @param x
	 *            Feld der L?nge n
	 */
	public double[] getValue(double[] x) {

		if (x == null || x.length != n)
			throw new IllegalArgumentException(Messages.getString(BUNDLE_NAME,
					"ParsedFunction.0")); //$NON-NLS-1$

		//Werte der Variablen setzen
		for (int i = 0; i < n; i++) {
			variables[i].setValue(x[i]);
		}

		//Auswerten der funktionen
		double[] y = new double[m];
		for (int i = 0; i < m; i++) {
			y[i] = functions[i].value();
		}

		return y;

	}

	/**
	 * Gibt einen String der Form (f1,f2,..,fm) zur?ck wobei fi f?r die
	 * Funktionen in der i-ten Komponente steht
	 *  
	 */
	public String toString() {
		if (functions == null || functions.length == 0)
			return ""; //$NON-NLS-1$

		int n = functions.length;

		if (n == 0)
			return functions[0].toString();
		else {
			String str = "("; //$NON-NLS-1$
			for (int i = 0; i < n; i++) {
				str += functions[i].toString();
				if (i < n - 1)
					str += ","; //$NON-NLS-1$

			}
			str += ")"; //$NON-NLS-1$

			return str;

		}

	}

}