package uibk.mtk.math.parsing;

import uibk.mtk.lang.InputException;
import uibk.mtk.lang.Messages;
import uibk.mtk.parser.realparser.Expr;
import uibk.mtk.parser.realparser.Parser;
import uibk.mtk.parser.realparser.SyntaxException;
import uibk.mtk.parser.realparser.Variable;

/**
 * Utilityklasse zum Parsen von reellen mathematischen Ausdr?cken/Funktionen
 * </CODE>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
final class RealExpressionParser {

	private static Parser parser = new Parser();

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.math.parsing.messages";//$NON-NLS-1$

	private RealExpressionParser() {
		super();

	}

	/**
	 * Parst die durch <CODE>strFunctions</CODE> definiert Funktion und gibt
	 * die so gewonnen Variablen und Ausdr?cke in einem Objekt der Klasse <CODE>
	 * ParsingResult</CODE> zur?ck
	 * 
	 * @param strVars
	 *            die Variablen
	 * @param strFunctions
	 *            mathematische Ausdr?cke in den Variablen
	 * @throws InputException
	 *             die vom Parser geworfenen Exceptions
	 */
	public static synchronized ParsingResult parseFunction(String[] strVars,
			String[] strFunctions) throws InputException {

		if (strVars == null || strVars.length == 0 || strFunctions == null
				|| strFunctions.length == 0)
			throw new IllegalArgumentException();

		//Variablen l?schen
		parser.clearVariables();

		//Anzahl Ortsvariablen
		int n = strVars.length;

		Variable[] variables = new Variable[n];

		//Variablen hinzuf?gen
		for (int i = 0; i < n; i++) {

			Variable v = new Variable(strVars[i]);
			variables[i] = v;
			parser.addVariable(v);
		}

		int m = strFunctions.length;
		Expr[] functions = new Expr[m];

		//Auswerten der Funktionsausdr?cke
		for (int i = 0; i < m; i++) {
			if (strFunctions[i] == null || strFunctions[i].equals("")) //$NON-NLS-1$
				if (m > 1)
					throw new InputException((i + 1)
							+ Messages.getString(BUNDLE_NAME,
									"RealExpressionParser.1")); //$NON-NLS-1$
				else
					throw new InputException(Messages.getString(BUNDLE_NAME,
							"RealExpressionParser.2")); //$NON-NLS-1$

			try {
				functions[i] = parser.parseString(strFunctions[i]);
			} catch (SyntaxException e) {
				throw new InputException(e.getMessage());
			}
		}

		//Setzen der Funktion
		return new ParsingResult(variables, functions);

	}

}