package uibk.mtk.swing.base;

import uibk.mtk.lang.InputException;
import uibk.mtk.math.functions.Function1D;
import uibk.mtk.math.parsing.Function1DParser;

/**
 * Ein <CODE>TextField</CODE> das eine Funktion zum Auswerten von Eingaben von
 * Intervallen bereitstellt.
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class Function1DTextField extends TextField {

	private String oldinput = null;

	private Function1D function = null;

	/** Variable in der die Funktion definiert sein muss */
	protected char var;

	/**
	 * Erzeugt das Textfeld
	 * 
	 * @param cols
	 *            Anzahl der Spalten des Textfeldes
	 * @param name
	 *            Name f?r Fehlermeldung
	 * @param realinterval
	 *            gibt an ob das Intervall ein echtes Intervall sein muss
	 *  
	 */
	public Function1DTextField(int cols, char var) {
		super(cols);

		this.var = var;

	}

	public boolean hasChanged() {
		return oldinput == null || !getText().equals(oldinput);
	}

	public void evaluateInput() throws InputException {

		if (hasChanged()) {
			function = Function1DParser.parse(String.valueOf(var), getText());
			oldinput = getText();

		}
	}

	/**
	 * Wertet die im Feld eingegebene Intervall aus gibt es zur?ck.
	 */
	public Function1D getFunction() throws InputException {
		evaluateInput();
		return function;
	}

}