package uibk.mtk.swing.scene3d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import uibk.mtk.draw3d.objects.surface3d.Surface3D;
import uibk.mtk.draw3d.base.MathPanel3D;
import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.TitledPanel;

/**
 * 
 * Panel zum Steuern der Sichtbarkeit fr zwei Fl?che (Surface3D)
 * 
 * <p>
 * Der Panel enthlt Optionsknpfe zum Wechseln der Sichtbarkeit
 * von zwei Flchen
 * </p>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2006
 * </p>
 * 
 * @author Rudigier Andreas (csae7164@uibk.ac.at)
 */
public class Panel2VisibleSurface  extends TitledPanel implements ActionListener {
	
	/** Flche deren Optionen gesetzt werden */
	Surface3D[] surface3d=new Surface3D[2];

	/** MathPanel der die <CODE>Scene3D</CODE> mit de Fl?che enth?tlt */
	MathPanel3D mathpanel3d;

	/** Option Drahtgitter */
	public JRadioButton radioswitch[]=new JRadioButton[2];

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

	/**
	 * Erzeugt den Panel
	 * 
	 * @param surface3d
	 *            Surface3D, deren Optionen eingestellt werden sollen
	 * @param mathpanel3d
	 *            MathPanel3D, der die Flche enthlt
	 */
	public Panel2VisibleSurface(Surface3D[] surface3d,MathPanel3D mathpanel3d) {
		super(Messages.getString(BUNDLE_NAME, "Panel2VisibleSurface.0")); //$NON-NLS-1$

		if (mathpanel3d == null)
			throw new NullPointerException(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.1")); //$NON-NLS-1$
		if (surface3d == null)
			throw new NullPointerException(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.2")); //$NON-NLS-1$
		this.surface3d = surface3d;
		this.mathpanel3d = mathpanel3d;
		initComponents();
	}

	/** Ereignisverarbeitung */
	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("switch1")) { //$NON-NLS-1$
			surface3d[1].setVisible(false);
			surface3d[0].setVisible(true);
			}
		if (event.getActionCommand().equals("switch2")) { //$NON-NLS-1$
			surface3d[0].setVisible(false);
			surface3d[1].setVisible(true);
		}
		mathpanel3d.requestRecompute();
		mathpanel3d.repaint();
	}

	/** Erzeugen der Komponenten des Panels */
	void initComponents() {

		//Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(1000, 130));

		//Erzeugen der Optionen fr die erste Flche
		ButtonGroup groupObenUnten = new ButtonGroup();
		radioswitch[0] = new JRadioButton(Messages.getString(BUNDLE_NAME,
				"Panel2VisibleSurface.1")); //$NON-NLS-1$
		radioswitch[0].setActionCommand("switch1"); //$NON-NLS-1$
		radioswitch[0].addActionListener(this);
		radioswitch[0].setSelected(true);

		//Erzeugen der Optionen fr die erste Flche
		radioswitch[1] = new JRadioButton(Messages.getString(BUNDLE_NAME,
				"Panel2VisibleSurface.2")); //$NON-NLS-1$
		radioswitch[1].setActionCommand("switch2"); //$NON-NLS-1$
		radioswitch[1].addActionListener(this);

		groupObenUnten.add(radioswitch[0]);
		groupObenUnten.add(radioswitch[1]);

		//hinzufgen der Optionen
		add(radioswitch[0], new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

		add(radioswitch[1], new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

	}


}
