package uibk.mtk.swing.scene3d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import uibk.mtk.draw3d.base.RMathPanel3D;
import uibk.mtk.draw3d.rasterizer.Rasterizer;
import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.TitledPanel;

/**
 * Panel zum Steuern Renderoptionen f?r eine Fl?che
 * 
 * <p>
 * Die einstellbaren Optionen sind "Qualit?t" und "Geschwindigkeit"
 * </p>
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class PanelRendering extends TitledPanel implements ActionListener {

	/** <CODE>RMathPanel3D</CODE> der die <CODE>Scene3D</CODE> enth?lt */
	RMathPanel3D mathpanel3d;

	/** <CODE>Rasterizer</CODE>, dessen Optionen gesetzt werden */
	Rasterizer rasterizer;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

	/**
	 * Erzeugt den Panel
	 * 
	 * @param mathpanel3d
	 *            RMathPanel3D f?r desen Rasterizer die Optionen gesetzt werden
	 */
	public PanelRendering(RMathPanel3D mathpanel3d) {
		super(Messages.getString(BUNDLE_NAME, "PanelRendering.0")); //$NON-NLS-1$
		this.mathpanel3d = mathpanel3d;
		this.rasterizer = mathpanel3d.getRasterizer();
		initComponents();
	}

	/** Ereignisverarbeitung: setzen der Optionen */
	public void actionPerformed(ActionEvent event) {

		if (event.getActionCommand().equals("speed")) { //$NON-NLS-1$
			rasterizer.setRenderingHint(Rasterizer.RENDER_HINT_SPEED);

		}
		if (event.getActionCommand().equals("quality")) { //$NON-NLS-1$
			rasterizer.setRenderingHint(Rasterizer.RENDER_HINT_QUALITY);
		}

		//neu zeichnen veranlassen
		mathpanel3d.repaint();
	}

	/** Initialisieren der Komponenten */
	void initComponents() {

		//Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(1000, 90));

		//Erzeugen der Optionen
		JRadioButton radioQuality = new JRadioButton(Messages.getString(
				BUNDLE_NAME, "PanelRendering.3")); //$NON-NLS-1$
		radioQuality.setActionCommand("quality"); //$NON-NLS-1$
		radioQuality.addActionListener(this);

		JRadioButton radioSpeed = new JRadioButton(Messages.getString(
				BUNDLE_NAME, "PanelRendering.5")); //$NON-NLS-1$
		radioSpeed.setActionCommand("speed"); //$NON-NLS-1$
		radioSpeed.addActionListener(this);

		ButtonGroup groupRenderingOpt = new ButtonGroup();

		groupRenderingOpt.add(radioQuality);
		groupRenderingOpt.add(radioSpeed);

		//aktuelle Einstellung auslesen
		if (rasterizer.getRenderingHint() == Rasterizer.RENDER_HINT_QUALITY) {
			radioQuality.setSelected(true);
		} else {
			radioSpeed.setSelected(true);
		}

		add(radioQuality, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						0, 0, 0), 0, 0));

		add(radioSpeed, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

	}

}