package uibk.mtk.util;

import uibk.mtk.draw2d.objects.DragablePoint2D;

/**
 * Utilityklasse die eine Funktion zum Erzeugen von Gitterpunkten in einem
 * Rechteck bereitstellt
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public final class Grid2Ddrag {

	private Grid2Ddrag() {
	}

	/**
	 * Erzeugt ein Gitter mit <CODE>gridx</CODE> x <CODE>gridy</CODE>
	 * Punkten und gibt sie in einem zweidimensionalen Array zur?ck. Die Punkte
	 * werden "spaltenweise" beginnend mit (xmin,ymin) angeordnet zur?ckgegeben.
	 * Falls xmax - xmin <= 0, ymax - ymin <= 0,gridx <2 oder gridy <2 wird eine
	 * <CODE>IllegalArgumentException</CODE> geworfen.
	 * 
	 * @param xmin
	 *            minimale x-Koordinate des Gitters
	 * @param xmax
	 *            maximale x-Koordinate des Gitters
	 * @param ymin
	 *            minimale y-Koordinate des Gitters
	 * @param ymax
	 *            maximale y-Koordinate des Gitters
	 * @param gridx
	 *            Anzahl der Gitterpunkte "parallel x-Achse"
	 * @param gridy
	 *            Anzahl der Gitterpunkte "parallel y-Achse"
	 * @return Gitter der Gr??e gridx x gridy
	 */
	public static DragablePoint2D[][] createGrid2D(double xmin, double xmax,
			double ymin, double ymax, int gridx, int gridy) {

		if (xmax - xmin <= 0 || ymax - ymin <= 0 || gridx < 2 || gridy < 2)
			throw new IllegalArgumentException();

		DragablePoint2D[][] gridpoints = new DragablePoint2D[gridx][gridy];

		IntervalIterator xIt = new IntervalIterator(xmin, xmax, gridx);
		IntervalIterator yIt = new IntervalIterator(ymin, ymax, gridy);

		double x, y;

		while (xIt.hasNext()) {
			x = xIt.nextdouble();

			while (yIt.hasNext()) {
				y = yIt.nextdouble();

				gridpoints[xIt.getPos() - 1][yIt.getPos() - 1] = new DragablePoint2D(x,
						y);

			}
			yIt.reset();

		}

		return gridpoints;
	}
}